#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.6.345 - update constants in getproductdefaults() to find installed setup config file

import sys
import os
import codecs
from fnmatch import fnmatch
import common as cf
from copy import deepcopy

__homedir__ = os.path.dirname(os.path.realpath(sys.argv[0]))
__homebase__ = os.path.basename(__homedir__)
__prefix__ = os.path.dirname(os.path.dirname(__homedir__))
__product__ = os.path.splitext(os.path.basename(sys.argv[0]))[0]

if os.name == 'nt':
	runas = os.getenv('USERNAME',os.getenv('USER','.'))
else:
	import pwd
	runas = os.environ['SUDO_USER'] if os.geteuid() == 0 and 'SUDO_USER' in os.environ else pwd.getpwuid(os.geteuid()).pw_name

defaults = {
	'setup': {
		'version': u'4.6.345',
		'product': __product__,
		'inifile': os.sep.join([__homedir__,''.join([__product__,'.ini'])]),
		'graphs': os.sep.join([__homedir__,'graphs']),
		'plugins': os.sep.join([__homedir__,'plugins']),
		'config-setup': '',
		'forcesetup': False,
		'active': '',
		'deletegraphs': False,
		'deleteplugins': False,
		},
	'filtergraph': {
		'rootfolder': os.sep.join([__prefix__,'share',__homebase__]) if os.path.exists(os.sep.join([__prefix__,'share',__homebase__])) else os.sep.join([__homedir__,'share']),
		},
	}

def getproductdefaults(testgraphs=True):
	global defaults

	basenameini = '%s.ini'%(__product__)
	prefixini = [__prefix__,'etc',]
	if os.name == 'nt':
		home = os.sep.join([os.getenv('HOMEDRIVE','.').strip(os.sep),os.getenv('HOMEPATH','.').strip(os.sep)]).lower()
		wine = os.sep.join(['z:','home',os.getenv('USERNAME',os.getenv('USER','.'))]).lower() # special wine testing
		host = os.sep.join([os.getenv('PROGRAMFILES','.'),__homebase__]).lower()
		installed_host = iniFileGet(os.sep.join([os.getenv('ALLUSERSPROFILE','.'),'Application Data','pttools','components-domt.ini']),__homebase__,'installdir',value = 'None')
		installed_user = iniFileGet(os.sep.join([os.getenv('APPDATA','.'),'pttools','components-domt.ini']),__homebase__,'installdir',value = 'None')
		if not installed_user.lower() == 'none' or \
			__homedir__.lower().startswith(home) or \
			__homedir__.lower().startswith(wine):
			# user installation
			prefixini = [os.getenv('APPDATA','.'),]
		elif not installed_host.lower() == 'none' or \
			__homedir__.lower().startswith(host):
			# host installation
			prefixini = [os.getenv('ALLUSERSPROFILE','.'),'Application Data',]

	if os.path.exists(os.sep.join(prefixini+[__homebase__,basenameini])):
		defaults['setup']['inifile'] = os.sep.join(prefixini+[__homebase__,basenameini])

	if testgraphs:
		if not os.path.exists(defaults['setup']['graphs']):
			raise RuntimeError("Error: corrupted installation - missing master graphs folder @ %s"%(defaults['setup']['graphs']))
		if not os.path.exists(defaults['setup']['plugins']):
			raise RuntimeError("Error: corrupted installation - missing master plugins folder @ %s"%(defaults['setup']['plugins']))

	return deepcopy(defaults)

def updateconfig(d,mastergraphs='',masterplugins=''):
	'''update config values from inifile'''

	# create default .ini file if missing
	_makedefaultini(defaults['setup']['inifile'],d)

	# set runas user
	lrunas = d['server']['runas'] if 'server' in d and 'runas' in d['server'] and d['server']['runas'] else runas

	# retrieve .ini file using runas value
	inistring = readrawinistring(expanduserdir(d['setup']['inifile'],lrunas))

	# update user options
	useropts = getuseroptions(inistring)
	inistring = setuseroptions(inistring,useropts)

	# convert to dictionary
	e = iniString2Dict(inistring)

	# update main config
	for k in e:
		if not k in d: d[k] = {}
		d[k].update(e[k])

	createdgraphs = False
	createdplugins = False

	if masterplugins and mastergraphs:

		workinggraphs = expanduserdir(d['setup']['graphs'],lrunas)
		workingplugins = expanduserdir(d['setup']['plugins'],lrunas)
		existedgraphs = os.path.exists(workinggraphs)
		existedplugins = os.path.exists(workingplugins)

		# copy master plugins plugins config folder
		_updatedir([masterplugins],workingplugins,lrunas,d['setup']['forcesetup'])

		# copy master "graphs" and others like "graphs-domy" and "graphs-domt"
		__prefix__, suffix = [ [x[0], '' ] if len(x) == 1 else [ x[0], '-%s'%(x[1]) ] for x in [os.path.basename(d['setup']['graphs']).split('-',1)]][0]
		_updatedir([mastergraphs,''.join([mastergraphs,suffix])],workinggraphs,lrunas,d['setup']['forcesetup'])

		createdgraphs = not existedgraphs and os.path.exists(workinggraphs)
		createdplugins = not existedplugins and os.path.exists(workingplugins)

	return createdgraphs, createdplugins

def getinstalledgraphs(graphs,graphpatterns=['*']):
	graphnames = []
	for graphpattern in graphpatterns:
		graphnames.extend([graphname for graphname in os.listdir(graphs) if fnmatch(graphname,graphpattern) and os.path.exists(os.sep.join([graphs,graphname,'config.ini']))])
	return sorted(cf.dedupelist(graphnames))

def readrawinistring(inifile):
	if not os.path.exists(inifile):
		return ''
	with codecs.open(inifile,'r','utf-8-sig') as f:
		content = f.read()
	return content

def writerawinistring(inifile,inistring):
	with codecs.open(inifile,'w','utf-8') as f:
		f.write(inistring)

def expanduserdir(path,lrunas=runas):
	# set "path" to proper user's $HOME folder when invoked as normal or super user
	if path.startswith('~'+os.sep) and os.name == 'posix' and os.geteuid() == 0:
		return path.replace('/',os.sep).replace('\\',os.sep).replace('~'+os.sep,pwd.getpwnam(lrunas).pw_dir+os.sep)
	else:
		return os.path.abspath(os.path.expanduser(path.replace('/',os.sep).replace('\\',os.sep)))

def getuseroptions(inistring):
	# strip ['USER'] section from inistring and convert to a list for search/replace
	isuser = False
	useropts = []
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines()]]:
		if line[0] == '[USER]':
			isuser = True
		elif line[0].startswith('[') and line[0].endswith(']') and isuser:
			break
		elif isuser and line[0] and not line[0].startswith(';') and not line[0].startswith('#'):
			useropts.append([line[0].lower(),line[1]])
	return useropts

def setuseroptions(inistring='',useropts=[]):
	config = []
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines()]]:
		if not line[0].strip().startswith(';') and not line[0].strip().startswith('#'):
			for i,j in useropts:
				line[0] = line[0].replace('%%(%s)s'%(unicode(i)),unicode(j))
				line[1] = line[1].replace('%%(%s)s'%(unicode(i)),unicode(j))
		config.append('='.join(line).rstrip('='))
	return '\x0d\x0a'.join(config)+'\x0d\x0a'

def Dict2iniString(config):
	inistring = []
	for k in sorted(config.keys()):
		inistring.append(k.join(['[',']']))
		for kk in sorted(config[k].keys()):
			inistring.append('='.join([unicode(kk),unicode(config[k][kk])]))
		inistring.append('')
	return '\x0d\x0a'.join(inistring)+'\x0d\x0a'

def iniString2Dict(inistring):
	config = {}
	section = ''
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines() if ('=' in line or (line.strip().startswith('[') and line.strip().endswith(']'))) and not line.strip().startswith(';') and not line.strip().startswith('#')]]:
		if line[0].startswith('[') and line[0].endswith(']'):
			section = line[0].strip('][')
			config[section] = {}
			continue
		if not section in config: continue
		config[section][line[0].lower()] = line[1]
	return config

def iniFileGet(inifile,section,key,value=''):
	if not os.path.exists(inifile):
		return value
	with codecs.open(inifile,'r','utf8') as f:
		inistring = f.read()
	searchsection = True
	for line in [[line[0].strip(),line[1].strip()] for line in [[line]+[''] if len(line.split('=',1)) < 2 else line.split('=',1) for line in inistring.splitlines() if ('=' in line or (line.strip().startswith('[') and line.strip().endswith(']'))) and not line.strip().startswith(';') and not line.strip().startswith('#')]]:
		if searchsection and not line[0] == section.join(['[',']']): continue
		searchsection = False
		if line[0] == key:
			value = '' if line[1].lower() == 'none' else line[1]
			break
		if line[0].startswith('[') and not line[0] == section.join(['[',']']) : break
	return value

def parseconfig(conf):
	errors = []
	cfg = {}
	config = deepcopy(conf)
	roottypes = ['BUILDS','CORPORA','ENGINES','RAW','TRAININGS','TRANSLATIONS','CORPORA.demo','RAW.demo',]

	# *** test for required sections ***
	if 'setup' in config:
		if not 'graphs' in config['setup']:
			errors.append([__name__,'missing','\"[setup] graphs=\" value not found'])
		if not 'plugins' in config['setup']:
			errors.append([__name__,'missing','\"[setup] plugins=\" value not found'])
	else:
		errors.append([__name__,'missing','\"[setup]\" section not found'])

	if not 'manager' in config:
		errors.append([__name__,'missing','\"[setup] graphs=\" value not found'])

	if 'filtergraph' in config:
		if not 'graphname' in config['filtergraph']:
			errors.append([__name__,'missing','\"[filtergraph] graphname=\" value not found'])
	else:
		errors.append([__name__,'missing','\"[filtergraph] section not found'])

	try:
		reader = ','.join([v.strip() for v in [k for k in config.keys() if ',reader-' in k.replace(' ','')][0].split(',')])
	except IndexError,e:
		errors.append([__name__,'missing','\"[reader]\" section not found'])
	else:
		if not 'stage' in config[reader]:
			errors.append([__name__,'missing','\"[%s] stage=\" value not found'%(reader)])
		if not 'roottype' in config[reader]:
			errors.append([__name__,'missing','\"[%s] roottype=\" value not found'%(reader)])
		if 'roottype' in config[reader] and not config[reader]['roottype'] in roottypes:
			errors.append([__name__,'invalid','\"[%s] roottype=\" value must be %s'%(reader,' or '.join(roottypes))])

	try:
		writer = ','.join([v.strip() for v in [k for k in config.keys() if ',writer-' in k.replace(' ','')][0].split(',')])
		if not 'stage' in config[writer]:
			errors.append([__name__,'missing','\"[%s] stage=\" value not found'%(writer)])
	except IndexError,e:
		errors.append([__name__,'missing','\"[writer]\" section not found'])
	else:
		if not 'roottype' in config[writer]:
			errors.append([__name__,'missing','\"[%s] roottype=\" value not found'%(writer)])
		if 'roottype' in config[reader] and not config[writer]['roottype'] in roottypes:
			errors.append([__name__,'invalid','\"[%s] roottype=\" value must be %s'%(writer,' or '.join(roottypes))])

	if errors: return errors

	# 0) get root [USER] 
	for k in [k for k in config.keys() if k == 'USER']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 1) get root [setup]
	for k in [k for k in config.keys() if k == 'setup']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 2) get root [manager] 
	for k in [k for k in config.keys() if k == 'manager']:
		cfg[k] = dict(config[k])
		del(config[k])
		break

	# 3) get [filtergraph]
	for k in [k for k in config.keys() if k == 'filtergraph']:
		cfg['manager'][k] = dict(config[k])
		del(config[k])
		cfg['manager'][k]['aligner'] = {}
		cfg['manager'][k]['filter'] = {}
		cfg['manager'][k]['postfilter'] = {}

		# test configuration keys for unset defaults
		tstkeys = ['stage','superdomains','domains','subdomains','corpustypes','sourcelanguages','languages','languagepairs','languagepairs','filespec','graphs','plugins','config-setup',]
		for tstkey in tstkeys:
			if tstkey in cfg['manager'][k] and '<USER-DEFINED>' in cfg['manager'][k][tstkey]:
				errors.append([__name__,'invalid','undefined value \"%s = <USER-DEFINED>\"'%(tstkey)])
				return errors

		cfg['manager'][k]['stage'] = cfg['manager'][k]['stage'].strip(',').split(',') if 'stage' in cfg['manager'][k] else []
		cfg['manager'][k]['superdomains'] = cfg['manager'][k]['superdomains'].strip(',').split(',') if 'superdomains' in cfg['manager'][k] else []
		cfg['manager'][k]['domains'] = cfg['manager'][k]['domains'].strip(',').split(',') if 'domains' in cfg['manager'][k] else []
		cfg['manager'][k]['subdomains'] = cfg['manager'][k]['subdomains'].strip(',').split(',') if 'subdomains' in cfg['manager'][k] else []
		cfg['manager'][k]['corpustypes'] = [l.strip() for l in cfg['manager'][k]['corpustypes'].strip(',').split(',') if l.strip().startswith('lm') or l.strip().startswith('tm') or l.strip() == '*'] if 'corpustypes' in cfg['manager'][k] else []
		cfg['manager'][k]['sourcelanguages'] = cfg['manager'][k]['sourcelanguages'].strip(',').split(',') if 'sourcelanguages' in cfg['manager'][k] else []
		cfg['manager'][k]['languages'] = cfg['manager'][k]['languages'].strip(',').split(',') if 'languages' in cfg['manager'][k] else []
		cfg['manager'][k]['languages'] = cf.dedupelist(cfg['manager'][k]['sourcelanguages']+cfg['manager'][k]['languages'])
		cfg['manager'][k]['languagepairs'] = [pair.strip(' -').strip().split('-') for pair in cfg['manager'][k]['languagepairs'].strip(',').split(',')] if 'languagepairs' in cfg['manager'][k] else []
		cfg['manager'][k]['languagepairs'] = [tuple([src.lower(),tgt.lower()]) for src,tgt in [pair if len(pair) == 2 else pair+pair for pair in cfg['manager'][k]['languagepairs']]]
		cfg['manager'][k]['filespec'] = cfg['manager'][k]['filespec'].strip(',').split(',') if 'filespec' in cfg['manager'][k] else []
		cfg['manager'][k]['graphs'] = cfg['manager'][k]['graphs'] if 'graphs' in cfg['manager'][k] and cfg['manager'][k]['graphs'] else cfg['setup']['graphs']
		cfg['manager'][k]['plugins'] = cfg['manager'][k]['plugins'] if 'plugins' in cfg['manager'][k] and cfg['manager'][k]['plugins'] else cfg['setup']['plugins']
		cfg['manager'][k]['config-setup'] = cfg['manager'][k]['config-setup'] if 'config-setup' in cfg['manager'][k] and cfg['manager'][k]['config-setup'] else cfg['setup']['config-setup']
		break

	# 4) get [reader]
	cfg['manager']['filtergraph']['reader'] = {reader: dict(config[reader]),}
	# parse known options to lists
	for k in [k for k in cfg['manager']['filtergraph']['reader'][reader].keys() if cf.isinteger(k[-1])]:
		if k[:4] == 'args' and cf.isinteger(k[4:]):
			cfg['manager']['filtergraph']['reader'][reader]['args'][int(k[4:])] = cfg['manager']['filtergraph']['reader'][reader][k]
			del(cfg['manager']['filtergraph']['reader'][reader][k])
		elif k[:7] == 'engines' and cf.isinteger(k[7:]):
			if not ',' in cfg['manager']['filtergraph']['reader'][reader][k]: continue
			cfg['manager']['filtergraph']['reader'][reader]['engines'][int(k[7:])] = cfg['manager']['filtergraph']['reader'][reader][k]
			del(cfg['manager']['filtergraph']['reader'][reader][k])
	del(config[reader])

	# 5) get [writer]
	cfg['manager']['filtergraph']['writer'] = {writer: dict(config[writer]),}
	# parse known options to lists
	for k in [k for k in cfg['manager']['filtergraph']['writer'][writer].keys() if cf.isinteger(k[-1])]:
		if k[:4] == 'args' and cf.isinteger(k[4:]):
			cfg['manager']['filtergraph']['writer'][writer]['args'][int(k[4:])] = cfg['manager']['filtergraph']['writer'][writer][k]
			del(cfg['manager']['filtergraph']['writer'][writer][k])
		elif k[:7] == 'engines' and cf.isinteger(k[7:]):
			if not ',' in cfg['manager']['filtergraph']['writer'][writer][k]: continue
			cfg['manager']['filtergraph']['writer'][writer]['engines'][int(k[7:])] = cfg['manager']['filtergraph']['writer'][writer][k]
			del(cfg['manager']['filtergraph']['writer'][writer][k])
	del(config[writer])

	# 6) fill plugin configurations
	for k in [k for k in config.keys() if len(k.split(',')) == 1 and not 'input' in k and not '.' in k]:
		lang = k.replace('-','_')
		for plugintype in ['aligner','filter','postfilter']:
			cfg['manager']['filtergraph'][plugintype][lang] = {}
			for kk in [kk for kk in config[k].keys() if kk.startswith(plugintype) and cf.isinteger(kk[len(plugintype):])]:
				if ','.join([k,kk.replace(plugintype,''),config[k][kk]]) in config.keys():
					cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))] = {config[k][kk]:config[','.join([k,kk.replace(plugintype,''),config[k][kk]])],}
					del(config[','.join([k,kk.replace(plugintype,''),config[k][kk]])])
					# parse known options to lists
					for kkk in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]].keys():
						if kkk in ['bufsize','close_fds','creationflags','cwd','env','executable','preexec_fn','shell','startupinfo','stderr','stdin','stdout','universal_newlines',]:
							if not 'kw' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['kw'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['kw'][kkk] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
						elif cf.isinteger(kkk[4:]) and kkk[:4] == 'args':
							if not 'args' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['args'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['args'][int(kkk[4:])] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
						elif cf.isinteger(kkk[7:]) and kkk[:7] == 'engines':
							if not ',' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]: continue
							if not 'engines' in cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]: cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['engines'] = {}
							cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]]['engines'][int(kkk[7:])] = cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk]
							del(cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))][config[k][kk]][kkk])
				else:
					cfg['manager']['filtergraph'][plugintype][lang][int(kk.replace(plugintype,''))] = {config[k][kk]:{},}
		del(config[k])

	# 7) add toolchain for other languages
	if 'languages' in cfg['manager']['filtergraph']:
		for k in cfg['manager']['filtergraph']['languages']:
			lang = '' if '*' in k or '?' in k else k.replace('-','_')
			for plugintype in ['aligner','filter','postfilter']:
				if lang and not lang in cfg['manager']['filtergraph'][plugintype]: cfg['manager']['filtergraph'][plugintype][lang] = {}

#	print _display(cfg)
#	exit()
	return cfg

def _display(cfg):
	for k in sorted(cfg.keys()):
		if type(cfg[k]) is dict:
			print unicode(k).join(['[',']'])
			for kk in sorted(cfg[k].keys()):
				if type(cfg[k][kk]) is dict:
					print unicode(kk).join(['     [',']'])
					for kkk in sorted(cfg[k][kk].keys()):
						if type(cfg[k][kk][kkk]) is dict:
							print unicode(kkk).join(['          [',']'])
							for kkkk in sorted(cfg[k][kk][kkk].keys()):
								if type(cfg[k][kk][kkk][kkkk]) is dict:
									print unicode(kkkk).join(['               [',']'])
									for kkkkk in sorted(cfg[k][kk][kkk][kkkk].keys()):
										if type(cfg[k][kk][kkk][kkkk][kkkkk]) is dict:
											print unicode(kkkkk).join(['                    [',']'])
											for kkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk].keys()):
												if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk]) is dict:
													print unicode(kkkkkk).join(['                         [',']'])
													for kkkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk].keys()):
														if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk]) is dict:
															print unicode(kkkkkkk).join(['                              [',']'])
															for kkkkkkkk in sorted(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk].keys()):
																if type(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk][kkkkkkkk]) is dict:
																	print unicode(kkkkkkkk).join(['                                   [',']'])
																else:
																	print '                                  ','='.join([unicode(kkkkkkkk),unicode(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk][kkkkkkkk])])
														else:
															print '                             ','='.join([unicode(kkkkkkk),unicode(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk][kkkkkkk])])
												else:
													print '                        ','='.join([unicode(kkkkkk),unicode(cfg[k][kk][kkk][kkkk][kkkkk][kkkkkk])])
										else:
											print '                   ','='.join([unicode(kkkkk),unicode(cfg[k][kk][kkk][kkkk][kkkkk])])
								else:
									print '              ','='.join([unicode(kkkk),unicode(cfg[k][kk][kkk][kkkk])])
						else:
							print '         ','='.join([unicode(kkk),unicode(cfg[k][kk][kkk])])
				else:
					print '    ','='.join([unicode(kk),unicode(cfg[k][kk])])
		else:
			print ' ','='.join([unicode(k),unicode(cfg[k])])
	print

def _updatedir(masters,working,lrunas,forcesetup):

	masters = [ master.rstrip(os.sep) for master in masters ]
	working = working.rstrip(os.sep)
	epoch = ['20000101T000000+0000', cf.str2datetime('20000101T000000+0000')]
	now = cf.timestamp(False).replace(' ','T').replace('-','').replace(':','').split('.')
	now = ['+'.join([now[0],now[1].split('+')[1]]),cf.str2datetime('+'.join([now[0],now[1].split('+')[1]]))]

	def get_lastupdate(path,datetime,createifmissing=False):
		try:
			with open(os.sep.join([path,'.lastupdate']),'r') as f:
				lines = f.read().splitlines()
		except:
			if createifmissing:
				try:
					os.makedirs(path)
				except OSError,e:
					if not e.errno == 17: raise OSError('Error: failed to create directory %s'%(path))
				try:
					with open(os.sep.join([path,'.lastupdate']),'w') as f:
						f.write('%s\n'%(datetime[0]))
				except:
					pass
			return list(datetime), {} # make_lastupdate(master,datetime)
		else:
			return [ lines[0], cf.str2datetime(lines[0]) ], dict([[os.sep.join([path,a[0]]),a[1]] for a in [path.split('\t') for path in lines[1:]]])

	# get date-time stamp from working folder
	working_lastupdate, working_contents = get_lastupdate(working,epoch)
	newest = list(working_lastupdate)

	# get to date-time stamps of master folders
	updates = {}
	for master in masters:

		if not os.path.isdir(master):
			if os.path.isdir(os.sep.join([os.path.dirname(master),'graphs-domt'])):
				master = os.sep.join([os.path.dirname(master),'graphs-domt'])
			elif os.path.isdir(os.sep.join([os.path.dirname(master),'graphs-domy'])):
				master = os.sep.join([os.path.dirname(master),'graphs-domy'])
			else:
				master = os.sep.join([os.path.dirname(master),'graphs'])

		master_lastupdate, master_contents = get_lastupdate(master,now,True)
		if master == working: continue

		# find the most recent update
		if master_lastupdate[1] > newest[1]: newest = list(master_lastupdate)

		# update working from master
		if (master_lastupdate[1] > working_lastupdate[1] and not master_lastupdate == now[0]) or forcesetup:
			_copytreeif(master,working,master_lastupdate[0],updates,working_contents,lrunas=lrunas)

	# write new working .lastupdate file if changes made
	if (updates and not master_lastupdate == now[0]) or forcesetup:
		contents = sorted(['%s\t%s'%(path.replace('%s%s'%(working,os.sep),''),crc) for path,crc in updates.items()])
		contents.insert(0,newest[0])
		try:
			with open(os.sep.join([working,'.lastupdate']),'w') as f:
				f.write('%s\n'%('\n'.join(contents)))
		except:
			pass

def _makedefaultini(inputini,config):
	if not 'setup' in config.keys() or not 'inifile' in config['setup'].keys():
		config = {}
		for k in defaults:
			config[k] = dict(defaults[k])
	lrunas = config['server']['runas'] if 'server' in config and 'runas' in config['server'] and config['server']['runas'] else runas
	outputini = expanduserdir(config['setup']['inifile'],lrunas)
	if os.path.exists(inputini) and not os.path.exists(outputini):
		content = [line.strip().replace('/',os.sep).replace('\\',os.sep) for line in codecs.open(inputini,'r','utf-8').readlines() if not line.strip().replace(' ','').startswith('inifile=')]
		# create output .ini file
		try:
			os.makedirs(os.path.dirname(outputini))
		except OSError,e:
			if not e.errno == 17: raise OSError('Error: failed to create directory %s'%(os.path.dirname(outputini)))
		try:
			codecs.open(outputini,'w','utf-8').write('\n'.join(content)+'\n')
		except:
			raise OSError('Error: failed to open %s'%(outputini))
		if os.name == 'posix' and os.geteuid() == 0:
			cf.chmodrecurse(os.path.dirname(working),pwd.getpwnam(lrunas).pw_uid,pwd.getpwnam(lrunas).pw_gid)

def _copytreeif(src, dst, datetime, updates={}, lastupdates={}, lrunas=''):
	"""Recursively copy a directory tree even if directory exists.
	shutil.copytree() modified with os.makedirs()"""
	import shutil
	errors = []

	if not os.path.isdir(src): return

	try:
		os.makedirs(dst)
	except OSError,e:
		if not e.errno == 17: errors.append('%s : %s : %s'%(src, dst, str(e)))

	for name in [name for name in os.listdir(src) if not name == '.lastupdate' and not name[-4:] == '.log' ]:
		srcname = os.path.join(src, name)
		dstname = os.path.join(dst, name)
		try:
			if os.path.isdir(srcname):
				_copytreeif(srcname, dstname, datetime, updates, lastupdates, lrunas)
			else:
				srccrc = str(cf.crc(srcname))
				if os.path.exists(dstname):
					dstcrc = str(cf.crc(dstname))
					if srccrc == dstcrc:
						updates[dstname] = dstcrc
					elif name == 'config.ini':
						altdestname = '%s.NEW-%s%s'%(os.path.splitext(dstname)[0],datetime,'.conf')
						shutil.copy2(srcname,altdestname)
#						updates[dstname] = str(cf.crc(srcname))
					else:
						if not dstname in lastupdates or not dstcrc == lastupdates[dstname]:
							# unregistered working copy, or working copy changed. rename working copy before coping new master
							altdestname = '%s.WORKING-%s%s.bak'%(os.path.splitext(dstname)[0],datetime,os.path.splitext(dstname)[1])
							shutil.move(dstname,altdestname)
						shutil.copy2(srcname,dstname)
						updates[dstname] = str(cf.crc(dstname))
				else:
					shutil.copy2(srcname,dstname)
					updates[dstname] = str(cf.crc(dstname))

		except Exception,e:
			errors.append('%s : %s : %s'%(srcname, dstname, str(e)))

	try:
		shutil.copystat(src, dst)
	except OSError,e:
		if WindowsError is not None and isinstance(e, WindowsError):
			# Copying file access times may fail on Windows
			pass
		else:
			errors.append('%s : %s : %s'%(src, dst, str(e)))

	if errors:
		raise Exception(errors)

	if os.name == 'posix' and os.geteuid() == 0 and lrunas:
		cf.chmodrecurse(dst,pwd.getpwnam(lrunas).pw_uid,pwd.getpwnam(lrunas).pw_gid)

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import parser\n\tfrom %s import manager\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	)

licensetxt=u'''CorpusFiltergraph™ v4.6
Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
