#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.6.344 - minor version update

import sys
import os
import logging
from glob import glob
import cfgparser
import manager
import common as cf

__version__ = '4.6.344'
createdgraphs = False
createdplugins = False

def main(graphconfigs,mergeoptions):
	'''Runs corpus filtgergraph manager'''

	result = [1,]

	# loop graphconfigs
	for config in graphconfigs:

		result[0] = 1

		# parse config to cfg
		cfg = cfgparser.parseconfig(config)

		if type(cfg) is list:

			result.extend(cfg)
			cfg = config

		else:

			# open graph with cfg
			mgr = open(cfg,mergeoptions,True)
			if type(mgr) is list:

				result.extend(mgr)

			else:

				if cfg['manager']['command'] == 'stop':

					# request running graph to stop
					result[0] = mgr.stop()

				elif cfg['manager']['command'] in ['run','restart']:

					# create queue
					if not mgr.cfg['restart']:
						result[0] = mgr.buildqueue()

					# run graph
					if mgr.cfg['usemanager'] and not result[0]:
						result[0] = mgr.run()

				# close manager (flush cumulative buffers)
				result.extend(mgr.close())

				# unload manager (delete plugin objects & unlock configuration)
				result.extend(mgr.unload())
				del(mgr)

		# report result - sample only. logging module also reports errors as the occur.
#		reporterrors(cfg,result[1:])

		reportresult(cfg,result[0])

		result[0] = 1 if result[0] or result[1:] else result[0]

		if result[0] > 0: break

	return result

def testgraphs(graphconfigs,mergeoptions):

	result = [0,]

	for config in graphconfigs:

		# parse config to cfg
		cfg = cfgparser.parseconfig(config)

		if type(cfg) is list:

			result.extend(cfg)
			cfg = config

		else:

			# open graph with cfg
			mgr = open(cfg,mergeoptions,False)
			if type(mgr) is list:

				result.extend(mgr)

			else:

				# close manager (flush cumulative buffers)
				mgr.close(False)

				# unload manager (delete plugin objects & unlock configuration)
				mgr.unload()
				del(mgr)

		reporterrors(cfg,result[1:])

		result[0] = 1 if result[0] or result[1:] else result[0]

	return result

def open(cfg,mergeoptions,verbose=True):

	errors = []

	# initialize manager
	try:
		mgr = manager.manager()
	except:
		errors.append([__name__,'failed','manager.manager() failed to load'])
		return errors

	# load configuration, and initialize graph with plugins
	errors.extend(mgr.load(cfg['manager']))

	# e.g. can call getcorpusattributes with options after loaded
#	for x in mgr.getcorpusattributes('/opt/domy','CORPORA','sa'):
#		print x
	# e.g. can call getallinputs with options after loaded
#	for k,v in mgr.getallinputs('/opt/domy','CORPORA','sa').items():
#		print k,v

	if not errors:

		# note: All subclasses have a cfg dict with default values.
		# For example, mgr.cfg, and mgr.filtergraph.cfg. The
		# function mgr.load(cfg['manager']) merges the config.ini
		# values with the default class values and returns
		# errors. An application can compare user command-line
		# options, cfg['manager'] values and class defaults.

		# review configuration and edit as needed (from global command line options)
		mergeoptions(cfg)

		# open graph (update all class values with cfg) & return errors
		errors.extend(mgr.open(cfg['manager'],verbose))

		# e.g. can call getcorpusattributes without options after opened
#		for x in mgr.getcorpusattributes():
#			print x
		# e.g. can call getallinputs without options after opened
#		for k,v in mgr.getallinputs().items():
#			print k,v

	if errors:
		mgr.close()
		mgr.unload()
		del(mgr)
		return errors

	return mgr

def reportresult(cfg,result):
	# parse result
	if result == 0:
		# x = '%s %s\t(success)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
		# sys.stderr.write(x.encode('utf8'))
		return
	elif result == 1:
		x ='%s %s\t(configuration failed)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 249:
		x = '%s %s\t(plugin runtime failed)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 250:
		x = '%s %s\t(API user interruption)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 251:
		x = '%s %s\t(Ctrl-C user interruption)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 252:
		x = '%s %s\t(undefined interruption)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 253:
		x = '%s %s\t(no files found)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 254:
		x = '%s %s\t(another instance running)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	elif result == 255:
		x = '%s %s\t(queue corrupted)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])
	else:
		x = '%s %s\t(unknown error)\n'%(os.path.splitext(os.path.basename(sys.argv[0]))[0],cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname'])

	sys.stderr.write(x.encode('utf8'))

def reporterrors(cfg,errors):
	try:
		if not cfg['manager']['debug']: return
	except:
		pass
	for err in errors:
		try:
			x = u'graphname: %s, module: %s, reports: %s, message: %s\n'%tuple([cfg['manager']['filtergraph']['graphname'] if 'filtergraph' in cfg['manager'] else cfg['filtergraph']['graphname']]+err)
			sys.stderr.write(x.encode('utf8'))
		except:
			pass

def runsetup():
	'''Runs corpus filtgergraph manager'''

	# get product defaults depending on platform and installation
	d = cfgparser.getproductdefaults()
	d['setup']['forcesetup'] = options.setup

	mastergraphs = d['setup']['graphs']
	masterplugins = d['setup']['plugins']

	# get host defaults
	cfgparser.updateconfig(d)

	# get user defaults
	global createdgraphs
	global createdplugins
	createdgraphs, createdplugins = cfgparser.updateconfig(d,mastergraphs,masterplugins)

	d['setup']['graphs'] = cfgparser.expanduserdir(d['setup']['graphs'])
	d['setup']['plugins'] = cfgparser.expanduserdir(d['setup']['plugins'])

	return d

def getgraph(graphname,defaults):
	'''gets configuration to run graphname'''

	# get inistring
	inistring = cfgparser.readrawinistring(os.sep.join([defaults['setup']['graphs'],graphname,'config.ini']))

	# get user options
	useropts = cfgparser.getuseroptions(inistring)

	# review & edit user options

	# update inistring with new changes
	inistring = cfgparser.setuseroptions(inistring,useropts)

	# convert inistring to config dict
	config = cfgparser.iniString2Dict(inistring)

	# merge config & inistring
	for k in defaults:
		if not k in config: config[k] = {}
		config[k].update(defaults[k])

	# update/add user & runtime values
	config['filtergraph']['rootfolder'] = cfgparser.expanduserdir(config['filtergraph']['rootfolder'])
	config['filtergraph']['graphname'] = graphname

	return config

def getallconfigs(graphs,graphname):
	'''returns *.ini contents in graph folder as list of strings'''
	import codecs
	allconfigs = []
	for inifile in glob(cfgparser.expanduserdir(os.sep.join([graphs,graphname,'*.ini']))):
		inilist = codecs.open(inifile,'r','utf8').read().splitlines()
		inilist.insert(0,os.path.basename(inifile).join(['[',']']))
		allconfigs.append('\x0a'.join(inilist))
	return allconfigs

def saveconfig(graphs,graphname,configname,content):
	'''save configname .ini file to graph folder'''
	import codecs
	f = codecs.open(os.sep.join([cfgparser.expanduserdir(graphs),graphname,configname]),'w','utf8')
	f.write('\x0a'.join(content.splitlines()))

def _mergeoptions(cfg):
	'''merge command line options with cfg'''
	if not options: return
	cfg['manager']['command'] = options.command if options.command else cfg['manager']['command']
	cfg['manager']['files'] = options.filename if options.filename else cfg['manager']['files']
	cfg['manager']['restart'] = options.restart if options.restart else cfg['manager']['restart']
	cfg['manager']['usemanager'] = not options.queue if options.queue else cfg['manager']['usemanager']
	cfg['manager']['debug'] = options.debug if options.debug else cfg['manager']['debug']
	cfg['manager']['verbose'] = logging.DEBUG if options.debug else cfg['manager']['verbose']
	cfg['manager']['cleanup'] = options.cleanup if options.cleanup else cfg['manager']['cleanup']
	cfg['manager']['markreadonly'] = options.markreadonly if options.markreadonly else cfg['manager']['markreadonly']
	cfg['manager']['resetreadonly'] = options.markreadonly if options.markreadonly else cfg['manager']['resetreadonly']
	cfg['manager']['versions'] = options.versions if options.versions else cfg['manager']['versions']

def _exit(returncode=0,pause=False):
	try:
		while pause:
			x = raw_input('Type "Ctrl-C" or "exit" <enter> to continue.\n')
			if x.lower().strip() == 'exit': pause = False
	except KeyboardInterrupt:
		pass
	sys.exit(returncode)

licensetxt=u'''CorpusFiltergraph™ v4.6
Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

abortmsg = '''
ABORT! You tried to run this program on a non-Windows host as the \"root\" 
user or the \"sudo\" command. This program runs many scripts created for 
academic research. Some of those scripts can cause damage to a host when 
run with \"root\" access rights. 

Therefore, this program has been disabled to run as \"root\". The manual 
has more information about how to configure your Linux host to access your 
data without running this program with \"root\" access. If you need more 
help, please contact Precision Translation Tools via email at 
sales@precisiontranslationtools.com for paid technical support options.\n\n'''

if __name__ == "__main__":

	from optparse import OptionParser

	try:

		# create log formatter
		formatter = logging.Formatter("%(asctime)s\t%(levelname)s\t%(name)s\t%(lineno)s\t%(funcName)s()\t%(message)s")
		# create logger
		logger = logging.getLogger(os.path.splitext(os.path.basename(sys.argv[0]))[0])
		logger.setLevel(logging.DEBUG)
		ch = logging.StreamHandler()
		ch.setLevel(logging.ERROR)
		ch.setFormatter(formatter)
		logger.addHandler(ch)

		# parse command line
		optparser = OptionParser(usage="usage: %prog [options] GRAPH1 [GRAPH2...]\n\nRun the graphs GRAPH1 and GRAPH2 with [options]")
		optparser.add_option("-r","--run",		action="store_const",	const="run",			dest="command",	default="run",	help="run GRAPH(s), default")
		optparser.add_option("-s","--stop",		action="store_const",	const="stop",			dest="command",	default="run",	help="stop GRAPH(s)")
		optparser.add_option("-f","--file",		action="append",		dest="filename",						default=[],		help="run input file(s) FILENAME")
		optparser.add_option("-P","--pause",	action="store_true",	dest="pause",							default=False,	help="pause to review results after all graphs finish")
		optparser.add_option("-q","--queue",	action="store_true",	dest="queue",							default=False,	help="build file queue and stop")
		optparser.add_option("-R","--restart",	action="store_true",	dest="restart",							default=False,	help="restart GRAPH(s) from previous queue")
		optparser.add_option("-m","--mark",		action="store_true",	dest="markreadonly",					default=False,	help="mark completed files read-only")
		optparser.add_option("-t","--reset",	action="store_true",	dest="resetreadonly",					default=False,	help="reset read-only files to read-write")
		optparser.add_option("-S","--setup",	action="store_true",	dest="setup",							default=False,	help="force configuration setup and exit")
		optparser.add_option("-C","--cleanup",	action="store_true",	dest="cleanup",							default=False,	help="delete lock file from previous interruption")
		optparser.add_option("-v","--verbose",	action="store_true",	dest="debug",							default=False,	help="display debug messages")
		optparser.add_option("-V","--versions",	action="store_true",	dest="versions",						default=False,	help="display versions of all modules")
		optparser.add_option("-l","--license",	action="store_true",	dest="license",							default=False,	help="display license information")

		options,graphnames = optparser.parse_args()

		if options.license:
			sys.stderr.write(licensetxt.encode('utf8')+'\n')
			_exit(0,options.pause)

		if os.name == 'posix' and  os.geteuid() == 0:
			sys.stderr.write(abortmsg)
			_exit(1,options.pause)

		defaults = runsetup()
		if options.setup:
			sys.stdout.write("Setup complete.\n")
			_exit(0,options.pause)

		# validate graphnames were requested
		graphnames = [graphname.strip('\\/,.') for graphname in graphnames]

		if options.versions:
			try:
				version = cfgparser.iniFileGet(defaults['setup']['config-setup'],'setup','version',value='')
			except:
				version = __version__
			sys.stderr.write('%s: %s\n'%(os.path.basename(sys.argv[0]),version))
			if not graphnames: _exit(1,options.pause)

		if not graphnames:
			optparser.error('\"%s\" missing operand: GRAPH1\n'%(os.path.basename(sys.argv[0])))

		# 1) get list of all graphs installed on system
		installedgraphs = cfgparser.getinstalledgraphs(defaults['setup']['graphs'],['*'])

		# 2) verify requested graphnames are installed
		missings = [graphname for graphname in graphnames if not graphname in installedgraphs]
		for missing in missings:
			optparser.error('missing - graph named \"%s\" not installed\n'%(missing))
		# NOTE: can also query available config options inside the graph with:
		#for graphname in graphnames:
		#	for graphconfig in getallconfigs(defaults['setup']['graphs'],graphname):
		#		print graphconfig

		# 3) retrieve graphs' config.ini, parse to dictionary, add to graphconfigs
		graphconfigs = [getgraph(graphname,defaults) for graphname in graphnames]

		# 4) test all graphconfigs before running
		result = testgraphs(graphconfigs,_mergeoptions)

	except Exception,e:

		if options.debug: logger.exception('')

		result = [1]

	if not result[0]:

		result = main(graphconfigs,_mergeoptions)

	import shutil
	if createdgraphs and defaults['setup']['deletegraphs']:
		try:
			shutil.rmtree(defaults['setup']['graphs'])
		except:
			pass
	if createdplugins and defaults['setup']['deleteplugins']:
		try:
			shutil.rmtree(defaults['setup']['plugins'])
		except:
			pass

	_exit(result[0],options.pause)
