#! /bin/bash
args=($*)
export LC_ALL=en_US.UTF-8

# prevent root from running this script
if [ $EUID -eq 0 ] ; then
	echo "" 1>&2
	echo "ABORT! You tried to run this program on a non-Windows host as the \"root\" " 1>&2
	echo "user or the \"sudo\" command. This program runs many scripts created for " 1>&2
	echo "academic research. Some of those scripts can cause damage to a host when " 1>&2
	echo "run with \"root\" access rights. " 1>&2
	echo "" 1>&2
	echo "Therefore, this program has been disabled to run as \"root\". The manual " 1>&2
	echo "has more information about how to configure your Linux host to access your " 1>&2
	echo "data without running this program with \"root\" access. If you need more " 1>&2
	echo "help, please contact Precision Translation Tools via email at " 1>&2
	echo "sales@precisiontranslationtools.com for paid technical support options." 1>&2
	echo "" 1>&2
	exit 1
	fi

usage(){
	cat <<- HELP
	Usage: ${0##*/} [OPTION...]
	Run the first demo wizard
	
	Options:
	  -c       continue without pausing to read prompts
	  -h       this help text
	  --clean  cleans demo output files
	
	Examples:
	  ${0##*/} -c
	     This command is useful to run all examples without pausing.
	
	HELP
	exit $1
	}

clean(){
	[ -d "$rootfolder/CORPORA/qc/Examples/Unicode/entities/lm/" ] && rm -rf "$rootfolder/CORPORA/qc/Examples/Unicode/entities/lm/"
	[ -d "$rootfolder/CORPORA/sa/Examples/News/general/tm/" ] && rm -rf "$rootfolder/CORPORA/sa/Examples/News/general/tm/"
	[ -d "$rootfolder/CORPORA/sa/European Commission/DGT_TM_1.0/Volume_1/tm/" ] && rm -rf "$rootfolder/CORPORA/sa/European Commission/DGT_TM_1.0/Volume_1/tm/"
	[ -d "$rootfolder/RAW/European Commission/DGT_TM_1.0/Volume_1/tm/" ] && rm -rf "$rootfolder/RAW/European Commission/DGT_TM_1.0/Volume_1/tm/"
	[ -d "$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm/" ] && rm -rf "$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm/"
	[ -d "$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm-dictionary/" ] && rm -rf "$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm-dictionary/"
	find "$rootfolder/CORPORA/sa/" -depth -type d -empty -exec rmdir {} \;
	find "$rootfolder/CORPORA/qc/" -depth -type d -empty -exec rmdir {} \;
	find "$rootfolder/RAW/" -depth -type d -empty -exec rmdir {} \;
	[ -f "${DOMY}/demo-champollion.sh" ] && "${DOMY}/demo-champollion.sh" $*
	exit 0
	}

DOMY=$(dirname `readlink -f ${0}`)
BASE=corpusfg

[[ "${args[@]}" =~ "-h" ]] && usage 0
[[ "${args[@]}" =~ "--help" ]] && usage 0

cfgm=${DOMY}/${BASE}
if [ ! -x "$cfgm" ] ; then
	echo "* $(date +%x\ %r) - ${0##*/} - Error: Please verify installation"
	echo "$cfgm not found."
	exit 1
	fi
[ -f "$HOME/domy/$BASE.ini" ] && userfiles=$HOME/domy
[ -f "$HOME/domt/$BASE.ini" ] && userfiles=$HOME/domt







# set graphs and rootfolder values using ini file if available
graphs=$userfiles/graphs
plugins=$userfiles/plugins
rootfolder=$(dirname $(dirname ${DOMY}))/share/corpusfg
if [ ! -d "$(dirname $(dirname ${DOMY}))/share/corpusfg" ] ; then
	rootfolder=$(dirname $(dirname ${DOMY}))/share/domy
	fi

if [[ "${args[@]}" =~ "--clean" ]] ; then
	clean $*
	fi

if [ -f "$userfiles/$BASE.ini" ] ; then
	while read line ; do
		[[ "${line}" =~ "=" ]] && eval "${line// /}" 2>/dev/null
		done < "$userfiles/$BASE.ini"
	fi

graphsexist=0
deletegraphs=0
pluginsexist=0
deleteplugins=0
[ -d "${graphs}" ] && graphsexist=1
[ -d "${plugins}" ] && pluginsexist=1
if [ $graphsexist -eq 0 -o $pluginsexist -eq 0 ] ; then
	[[ "${args[@]}" =~ "-c" ]] && echo -e "* $(date +%x\ %r) - ${0##*/} - stop - can not run with -c option\nthe because \"${graphs##*/}\" or \"${plugins##*/}\" folder is missing. You can run \"${0##*/}\" \nwithout \"-c\", or run \"${BASE/.py/} -S\" to reset your configuration before\nrunning with the \"-c\" option." && exit 1
	"$cfgm" -vS
	if [ $? -ne 0 ] ; then echo "* $(date +%x\ %r) - ${0##*/} - configuration failure - reinstallation required" ; exit 1 ; fi
	fi
[ -d "${graphs}" -a ${graphsexist} -eq 0 ] && deletegraphs=1
[ -d "${plugins}" -a ${pluginsexist} -eq 0 ] && deleteplugins=1

leave(){
	[ ${deletegraphs} -eq 1 ] && rm -r "${graphs}"
	[ ${deleteplugins} -eq 1 ] && rm -r "${plugins}"
	[[ $1 -eq 255 ]] && echo
	exit $1
	}
trap "leave 255" SIGINT

clear

echo "Introduction:"
echo
echo "This is the first of two demos. This demo runs five (5) example"
echo "\"GRAPHS\". GRAPHS are parallel tool-chains that extract, clean,"
echo "align parallel text, and otherwise prepare corpora and translation"
echo "memory data for use as SMT training data."
echo
echo "Example 1 converts HTML entities to UTF-8 characters"
echo "Example 2 creates sentence-aligned files from file-aligned data"
echo "Example 3 imports data from TMX files to CorpusFiltergraph"
echo "Example 4 exports data from CorpusFiltergraph to TMX files"
echo "Example 5 extracts bilingual data from individual files"
echo
echo "You can find the GRAPHs and their config.ini files in:"
echo "$graphs"
echo

echo "Next: Instructions"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Instructions:"
echo
echo "This is a wizard-style demo. It pauses before running an example"
echo "with a short description and the GRAPH's command line."
echo
echo "It pauses after running the example to show the command line,"
echo "the input and output of the example. Right-click the input or"
echo "output path and select \"Open Link\" to see the changes."
echo




echo "Next: Example 1, clean HTML/XML markup"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 1: clean HTML/XML markup"
echo
echo "Cleaning HTML/XML data is an important task for all data preparation."
echo "HTML and XML documents often include \"entities\" that represent characters."
echo "For example, &lt; is \"<\" and &gt; is \">\" and &aacute; is \"á\". This GRAPH"
echo "converts these \"entities\" back to \"normal\" characters."
echo
echo "Command:  ~\$ $BASE clean-markup"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 1..."
clear

echo "Example 1: clean HTML/XML markup"
echo
if [ -f "$graphs/clean-markup/Demo - clean html xml entities.conf" ]; then
	cp "$graphs/clean-markup/config.ini" $graphs/clean-markup/.config.ini.tmp
	cp "$graphs/clean-markup/Demo - clean html xml entities.conf" $graphs/clean-markup/config.ini
	python $cfgm -v clean-markup
	result=$?
	mv $graphs/clean-markup/.config.ini.tmp $graphs/clean-markup/config.ini
	[ $result -ne 0 ] && leave $result
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE clean-markup"
	echo "Input:    \"file://$rootfolder/CORPORA/sa/Examples/Unicode/entities/lm/en/en/character-entities.txt\""
	echo "Output:   \"file://$rootfolder/CORPORA/qc/Examples/Unicode/entities/lm/en/en/character-entities.txt\""
	echo
else
	echo "\"clean-markup\" demo configuration is not installed."
	echo
	fi

if [ -f "${DOMY}/demo-champollion.sh" ] ; then
	echo "Next: Example 2, create sentence-aligned data from aligned files"
	[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
	clear
	"${DOMY}/demo-champollion.sh" $*
	[ $? -ne 0 ] && leave $result
	fi

echo "Next: Example 3, import data from TMX files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 3: import data TMX files"
echo
echo "TMX (translation memory exchange) files were developed to exchange"
echo "translation memory data between companies, colleagues and systems."
echo "This GRAPH imports aligned tmx data into the CorpusFiltergraph"
echo "folder hierarchy. These sample TMX files come from the DGT-TM"
echo "version 1 and maintain sentence alignment across 13 languages."
echo
echo "Command:  ~\$ $BASE import-tmx"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 3..."
clear

echo "Example 3: import data TMX files"
echo
echo "Command:  ~\$ $BASE import-tmx"
echo
if [ -f "$graphs/import-tmx/Demo - import DGT TM 1.0 TMX files to corpus.conf" ]; then
	cp "$graphs/import-tmx/config.ini" $graphs/import-tmx/.config.ini.tmp
	cp "$graphs/import-tmx/Demo - import DGT TM 1.0 TMX files to corpus.conf" $graphs/import-tmx/config.ini
	python $cfgm -v import-tmx
	result=$?
	mv $graphs/import-tmx/.config.ini.tmp $graphs/import-tmx/config.ini
	[ $result -ne 0 ] && leave $result
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE import-tmx"
	echo "Input1:   \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/21985A0705(01).tmx\""
	echo "Input2:   \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/22001D0118(18).tmx\""
	echo "Output1:  \"file://$rootfolder/CORPORA/sa/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/\""
	echo
else
	echo "\"import-tmx\" demo configuration is not installed."
	echo
	fi

echo "Next: Example 4, export data to TMX files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 4: export data to TMX files"
echo
echo "This GRAPH exports tm data to TMX files. Data from the CorpusFiltergarph"
echo "folder hierarchy can be shared with other translation systems."
echo
echo "Command:  ~\$ $BASE export-tmx"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 4..."
clear

echo "Example 4: export data to TMX files"
echo
echo "Command:  ~\$ $BASE export-tmx"
echo
if [ -f "$graphs/export-tmx/Demo - export corpora to TMX files.conf" ]; then
	cp "$graphs/export-tmx/config.ini" $graphs/export-tmx/.config.ini.tmp
	cp "$graphs/export-tmx/Demo - export corpora to TMX files.conf" $graphs/export-tmx/config.ini
	python $cfgm -v export-tmx
	result=$?
	mv $graphs/export-tmx/.config.ini.tmp $graphs/export-tmx/config.ini
	[ $result -ne 0 ] && leave $result
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE export-tmx"
	echo "Inputs:   \"file://$rootfolder/CORPORA/sa/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/\""
	echo "Output1:  \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/21985A0705(01).tmx.txt.tmx\""
	echo "Output2:  \"file://$rootfolder/RAW/European%20Commission/DGT_TM_1.0/Volume_1/tm/en_gb/22001D0118(18).tmx.txt.tmx\""
	echo
else
	echo "\"export-tmx\" demo configuration is not installed."
	echo
	fi

echo "Next: Example 5, extract bilingual dictionaries from individual files"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Example 5: extract bilingual dictionaries from individual files"
echo
echo "This is an example of a custom GRAPH. Users and PTTools can create"
echo "custom GRAPHS for special data-cleaning purposes. This example's"
echo "input file has parenthetical entries in-line with the main text."
echo "This GRAPH removes the parenthetical entries from the original files"
echo "and saves them as new parallel dictionary files."
echo
echo "Command:  ~\$ $BASE extract-dictionary"
echo
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" to start Example 5..."
clear

echo "Example 5: extract bilingual dictionaries from individual files"
echo
echo "Command:  ~\$ $BASE extract-dictionary"
echo
if [ -f "$graphs/extract-dictionary/Demo - extract custom dictionaries from zh_uk-en corpus.conf" ]; then
	cp "$graphs/extract-dictionary/config.ini" $graphs/extract-dictionary/.config.ini.tmp
	cp "$graphs/extract-dictionary/Demo - extract custom dictionaries from zh_uk-en corpus.conf" $graphs/extract-dictionary/config.ini
	python $cfgm -v extract-dictionary
	result=$?
	mv $graphs/extract-dictionary/.config.ini.tmp $graphs/extract-dictionary/config.ini
	[ $result -ne 0 ] && leave $result
	echo
	echo "Review"
	echo "Command:  ~\$ $BASE extract-dictionary"
	echo "Input1:   \"file://$rootfolder/CORPORA/sa/Examples/Legislation/statutes/tm/zh_hk/zh_hk/hklaws.txt\""
	echo "Input2:   \"file://$rootfolder/CORPORA/sa/Examples/Legislation/statutes/tm/zh_hk/en/hklaws.txt\""
	echo "Outputs:  \"file://$rootfolder/CORPORA/qc/Examples/Legislation/statutes/\""
	echo "Output1:  \"file://$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm/zh_hk/zh_hk/hklaws.txt\""
	echo "Output2:  \"file://$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm/zh_hk/en/hklaws.txt\""
	echo "Output3:  \"file://$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm-dictionary/zh_hk/zh_hk/hklaws.txt\""
	echo "Output4:  \"file://$rootfolder/CORPORA/qc/Examples/Legislation/statutes/tm-dictionary/zh_hk/en/hklaws.txt\""
	echo
else
	echo "\"extract-dictionary\" demo configuration is not installed."
	echo
	fi

echo "Next: Review"
[[ ! "${args[@]}" =~ "-c" ]] && read -n 1 -p "Press \"Enter\" continue..."
clear

echo "Review:"
echo
echo "Examples in Demo-1.sh extracted, cleaned and aligned parallel text data."
echo
echo "Example 1 converted HTML entities to UTF-8 characters"
echo "Example 2 created sentence-aligned files from file-aligned data"
echo "Example 3 imported data from TMX files to CorpusFiltergraph"
echo "Example 4 exported data from CorpusFiltergraph to TMX files"
echo "Example 5 extracted bilingual data from individual files"
echo
echo "NOTE: All outputs were saved to sub-folders in"
echo "  \"$rootfolder/CORPORA/*\" or"
echo "  \"$rootfolder/RAW/*\"."
echo "  The examples in the next demo, \"demo-2.sh\" in DoMY CE, continue"
echo "  with the outputs from this demo. To delete these demo files after"
echo "  running \"demo-2.sh\" in DoMY CE, run \"demo-1.sh -clean\"."
echo
echo "Demo-2.sh prepares two data BUILD sets from the cleaned data and"
echo "trains two translation engines. You can start then DoMY CE demo"
echo "with the command \"demo-2.sh\""
echo

echo "Next: Repeat demo-1.sh or run demo-2.sh (requires DoMY CE)"

leave 0

#CorpusFiltergraph™ v4.6.344
#Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU Lesser General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public License
#along with this program.  If not, see http://www.gnu.org/licenses/.

#For more information, please contact Precision Translation Tools Co., Ltd.
#at: http://www.precisiontranslationtools.com
