#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Tom Hoar
#===============================================================================

#version:
#4.6.344 - minor version update

import sys
import os
import shutil
import imp
import logging
import flock
import fnmatch
import stat
import common as cf
import tempfile

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager',__name__]))
plugintypes = ['filter','aligner','postfilter']

class filtergraph(object):
	'''filtergraph class builds filter and aligner graphs per language'''

	class CustomInput(dict):
		'''custom file object dictionary'''
		def __getitem__(self,key):
			try:
				return super(filtergraph.CustomInput,self).__getitem__(key)
			except KeyError:
				if me.stage[0].lower() == 'none':
					filelist = me.filelist[:cf.stage]+list(key)
				else:
					filelist = me.filelist[:cf.stage]+[me.filelist[cf.stage]]+list(key)
				self[key] = {
					'filelist': filelist,
					'tempname': cf.tempname(me.filelist[:cf.stage]+[me.filelist[cf.stage]]+list(key),'~','.tmp'),
					'tempbuff': [],
					'tempobj': None,
					'lock': None,
					}
				if not os.path.exists(os.sep.join(self[key]['filelist'])):
					logger.warn('%s\t%s',*['missing',os.sep.join(self[key]['filelist'])])
				elif not os.access(os.sep.join(self[key]['filelist']),os.W_OK):
					logger.warn('%s\t%s',*['read-only',os.sep.join(self[key]['filelist'])])
				elif flock.flock(os.extsep.join([os.sep.join(self[key]['filelist']),'lock']),False).islocked():
					logger.warn('%s\t%s',*['in-use',os.sep.join(self[key]['filelist'])])
				else:
					self[key]['lock'] = flock.flock(os.extsep.join([os.sep.join(self[key]['filelist']),'lock']),False).acquire()
					logger.debug('%s\t%s',*['locked',os.sep.join(self[key]['filelist'])])
				return super(filtergraph.CustomInput,self).__getitem__(key)
		def __delitem__(self,key):
			if me.markreadonly: me.reader.flagrecord(os.sep.join(self[key]['filelist']))
			try:
				self[key]['lock'].release()
				self[key]['lock'] = None
				logger.debug("%s\t%s",*['released',os.sep.join(self[key]['filelist'])])
			except:
				pass
			try:
				os.unlink(self[key]['tempname'])
			except:
				pass
			return super(filtergraph.CustomInput,self).__delitem__(key)

	class CustomOutput(dict):
		'''custom file object dictionary'''
		def __getitem__(self,key):
			try:
				return super(filtergraph.CustomOutput,self).__getitem__(key)
			except KeyError:
				fs,root = me.writer.getroot(me.writer.rootfolder) if me.writer.rootfolder else me.filelist[:cf.rttype]
				if me.writer.stage.lower() == 'none':
					filelist = [fs,root]+[me.writer.roottype]+list(key)
				else:
					stage = me.filelist[cf.stage] if fnmatch.fnmatch(me.filelist[cf.stage],me.writer.stage) else me.writer.stage
					filelist = [fs,root]+[me.writer.roottype]+[stage]+list(key)
				self[key] = {
					'filelist': filelist,
					'tempname': cf.tempname(filelist,'~','.tmp'),
					'tempbuff': [],
					'tempobj': None,
					'lock': None,
					}
				if os.path.exists(os.sep.join(self[key]['filelist'])) and me.writer.safemode:
					logger.warn('%s\t%s',*['safe-mode',self[key]['filelist']])
				elif flock.flock(os.extsep.join([self[key]['tempname'],'lock']),False).islocked():
					logger.warn('%s\t%s',*['in-use',self[key]['tempname']])
				else:
					self[key]['lock'] = flock.flock(os.extsep.join([self[key]['tempname'],'lock']),False).acquire()
					logger.debug('%s\t%s',*['locked',self[key]['tempname']])
					me.cferror[key]['tempbuff'] = []
				return super(filtergraph.CustomOutput,self).__getitem__(key)
		def __delitem__(self,key):
			try:
				self[key]['lock'].release()
				self[key]['lock'] = None
			except:
				pass
			try:
				os.unlink(self[key]['tempname'])
				logger.debug("%s\t%s",*['released',self[key]['tempname']])
			except:
				pass
			return super(filtergraph.CustomOutput,self).__delitem__(key)

	class CustomError(dict):
		'''custom file object dictionary'''
		def __getitem__(self,key):
			try:
				return super(filtergraph.CustomError,self).__getitem__(key)
			except KeyError:
				fs,root = me.writer.getroot(me.writer.rootfolder) if me.writer.rootfolder else me.filelist[:cf.rttype]
				if me.writer.stage.lower() == 'none':
					filelist = [fs,root]+[me.writer.roottype]+['-'.join([key[0],'workbench'])]+list(key[1:])
				else:
					stage = me.filelist[cf.stage] if fnmatch.fnmatch(me.filelist[cf.stage],me.writer.stage) else me.writer.stage
					filelist = [fs,root]+[me.writer.roottype]+['-'.join([stage,'workbench'])]+list(key)
				self[key] = {
					'filelist': filelist,
					'tempname': cf.tempname(filelist,'~','.tmp'),
					'tempbuff': [],
					'tempobj': None,
					'lock': None,
					}
				self[key]['lock'] = flock.flock(os.extsep.join([self[key]['tempname'],'lock']),False).acquire()
				logger.debug('%s\t%s',*['locked',self[key]['tempname']])
				return super(filtergraph.CustomError,self).__getitem__(key)
		def __delitem__(self,key):
			try:
				self[key]['lock'].release()
				self[key]['lock'] = None
			except:
				pass
			try:
				os.unlink(self[key]['tempname'])
				logger.debug("%s\t%s",*['released',self[key]['tempname']])
			except:
				pass
			return super(filtergraph.CustomError,self).__delitem__(key)

	class CustomExtract(dict):
		'''custom file object dictionary'''
		def __getitem__(self,key):
			try:
				return super(filtergraph.CustomExtract,self).__getitem__(key)
			except KeyError:
				fs,root = me.writer.getroot(me.writer.rootfolder) if me.writer.rootfolder else me.filelist[:cf.rttype]
				stage = me.filelist[cf.stage] if fnmatch.fnmatch(me.filelist[cf.stage],me.writer.stage) else me.writer.stage
				filelist = [fs,root]+[me.writer.roottype]+['-'.join([stage,'extract'])]+list(key)
				self[key] = {
					'filelist': filelist,
					'tempname': cf.tempname(filelist,'~','.tmp'),
					'tempbuff': [],
					'tempobj': None,
					'lock': None,
					}
				self[key]['lock'] = flock.flock(os.extsep.join([self[key]['tempname'],'lock']),False).acquire()
				logger.debug('%s\t%s',*['locked',self[key]['tempname']])
				return super(filtergraph.CustomExtract,self).__getitem__(key)
		def __delitem__(self,key):
			try:
				self[key]['lock'].release()
				self[key]['lock'] = None
			except:
				pass
			try:
				os.unlink(self[key]['tempname'])
				logger.debug("%s\t%s",*['released',self[key]['tempname']])
			except:
				pass
			return super(filtergraph.CustomExtract,self).__delitem__(key)

	class reader(object):
		cfg = {
			'encoding': 'utf8',
			'maxinput': 0,
			'null': False,
			'roottype': '',
			'stage': None,
			'version': '',
			}
		encoding = 'utf8'
		maxinput = 0
		null = False
		roottype = ''
		seen = {}
		stage = None
		errors = []
		p = object
		isopen = False
		def open(self,parent,cfg):
			return
		def run(self):
			return
		def flush(self):
			return
		def close(self):
			return
		def getrecord(self):
			return
		def flagrecord(self,filelist):
			return
		def unflagrecord(self,filelist):
			return
		def buildqueue(self,srcqueue,searchpaths):
			return
		def path2list(self,path,root):
			return
		def getroot(self,rootfolder):
			return
		def normalizeroot(self,rootfolder):
			return

	class writer(object):
		cfg = {
			'append': False,
			'deletezerolength': False,
			'encoding': 'utf8',
			'eol': 'linux',
			'null': False,
			'rendertypes': '.*',
			'rootfolder': '',
			'roottype': '',
			'safemode': False,
			'stage' : None,
			'version': '',
			}
		append = 'w'
		deletezerolength = False
		encoding = 'utf8'
		eol = '\x0a'
		null = False
		rendertypes = '.*'
		rootfolder = ''
		roottype = ''
		safemode = False
		stage = None
		errors = []
		p = object
		isopen = False
		def open(self,parent,cfg):
			return
		def run(self,fileobj,k=None,mode=None):
			return
		def flush(self,fileobj,k=None):
			return
		def close(self):
			return

		def getroot(self,rootfolder):
			return
		def normalizeroot(self,rootfolder):
			return

	class filter(object):
		cfg = {
			'version': '',
			}
		encoding = 'utf8'
		errors = []
		p = object
		isopen = False
		def open(self,parent,cfg):
			return
		def run(self,k):
			return
		def flush(self,k):
			return
		def close(self):
			return

	class aligner(object):
		cfg = {
			'version': '',
			}
		errors = []
		p = object
		isopen = False
		def open(self,parent,cfg):
			return
		def run(self,ktgt):
			return
		def flush(self,ktgt):
			return
		def close(self):
			return

	cfg = {
		'bypass': False,
		'corpustypes': [],
		'distinct-flush-runs': False,
		'domains': [],
		'filespec': ['*.*'],
		'graphname': None,
		'languagepairs': [],
		'languages': [],
		'lmonly': False,
		'queueby': 'source',
		'reader': {},
		'rootfolder': None,
		'sourcelanguages': [],
		'stage': [],
		'subdomains': [],
		'superdomains': [],
		'writer': {},
		'version': '4.6.344',
		'versions': False,
		}
	bypass = False
	cferror = CustomError()
	cfextract = CustomExtract()
	cfinput = CustomInput()
	cfoutput = CustomOutput()
	corpustypes = []
	datetime = ''
	debug = False
	distinct_flush_runs = False
	domains = []
	filelist = []
	filespec = ['*.*']
	graphname = ''
	graphs = ''
	languagepairs = []
	languagepatterns = []
	languages = []
	lmonly = False
	plugins = ''
	queuecount = 0
	queueby = 'source'
	reader = reader()
	rootfolder = ''
	sourcelanguages = []
	sourcepatterns = []
	srcqueue = ''
	stage = []
	statusgraph = ''
	statusplugin = ''
	subdomains = []
	superdomains = []
	targetpatterns = []
#	if os.name == 'nt':
#		tempdir = os.sep.join([os.environ['TEMP'],os.path.splitext(os.path.basename(sys.argv[0]))[0], if os.path.splitext(os.path.basename(sys.argv[0]))[0] else 'domy'])
#	else:
	tempdir = os.sep.join([tempfile.gettempdir(),os.path.splitext(os.path.basename(sys.argv[0]))[0] if os.path.splitext(os.path.basename(sys.argv[0]))[0] else 'domy'])

	toolchain = {plugintypes[0]:{},plugintypes[1]:{},plugintypes[2]:{}}
	verbose = logging.WARNING
	writer = writer()
	isrunning = False
	isloaded = False
	errors = []
	isopen = False

	def load(self,cfg):

		self.errors = []

		if self.isloaded:
			self.errors.append([__name__,'skip','already loaded'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		# validate required defaults & merge configuration
		for k in self.cfg.keys():
			if k in cfg and cfg[k]: continue
			cfg[k] = self.cfg[k]
			if cfg[k] is None:
				self.errors.append([__name__,'missing','[%s] \"%s=\"'%(__name__,k)])
				logger.error('%s\t%s',*self.errors[-1][1:])
		if cfg['corpustypes'] and not cfg['languagepairs']:
			for ct in cfg['corpustypes']:
				self.errors.append([__name__,'missing','[%s] \"corpustypes=%s\" requires \"languagepairs=<missing>\"'%(__name__,ct)])
				logger.error('%s\t%s',*self.errors[-1][1:])

		# convert filtergraph config values to proper types
		self.__setDataTypes(self.cfg,cfg)

		# exit if self.errors
		if self.errors:
			self.isloaded = not self.errors
			return self.errors

		global me
		me = self

		self.graphname = cfg['graphname']
		self.graphs = os.path.abspath(os.path.expanduser(cfg['graphs']))
		self.plugins = os.path.abspath(os.path.expanduser(cfg['plugins']))
		self.rootfolder = os.path.abspath(os.path.expanduser(cfg['rootfolder']))
		self.config_setup = cfg['config-setup']

		# create input dictionary
		self.cfinput = self.CustomInput()
		# create output dictionaries
		self.cfoutput = self.CustomOutput()
		self.cferror = self.CustomError()
		self.cfextract = self.CustomExtract()

		# tempdir required before initializing plugins
		self.tempdir = os.sep.join([self.tempdir,''.join(['~',self.graphname])])
		if not os.path.exists(self.tempdir):
			try:
				os.makedirs(self.tempdir)
			except OSError,e:
				if not e.errno == 17:
					logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
					raise OSError(e)

		sys.path.insert(0,self.tempdir)

		# initialize reader plugin
		plugintype = 'reader'
		module = cfg[plugintype].keys()[0]
		self.reader = self.getplugin(plugintype,'--',0,module)
		if self.reader:
			self.reader.errors = []
			self.reader.p = self
			# merge defaults with configuration
			for k in self.reader.cfg.keys():
				if k in cfg[plugintype][module] and cfg[plugintype][module][k]: continue
				cfg[plugintype][module][k] = self.reader.cfg[k]
				if cfg[plugintype][module][k] is None:
					self.errors.append([module,'missing','[%s] \"%s=\"'%(module,k)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				elif k[:4] == 'args' and cf.isinteger(k[4:]):
					if not 'args' in cfg[plugintype][module]: cfg[plugintype][module]['args'] = {}
					cfg[plugintype][module]['args'][int(k[4:])] = cfg[plugintype][module][k]
					del(cfg[plugintype][module][k])
				elif k[:7] == 'engines' and cf.isinteger(k[7:]):
					if not ',' in cfg[plugintype][module][k]: continue
					if not 'engines' in cfg[plugintype][module]: cfg[plugintype][module]['engines'] = {}
					cfg[plugintype][module]['engines'][int(k[7:])] = cfg[plugintype][module][k]
					del(cfg[plugintype][module][k])
			# convert reader config values to proper types
			cfg[plugintype][module]['stage'] = [item.strip() for item in cfg[plugintype][module]['stage'].strip(', ').split(',')]
			self.__setDataTypes(self.reader.cfg,cfg[plugintype][module])
		else:
			self.errors.append([module,'failed','%s'%(module)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# initialize writer plugin
		plugintype = 'writer'
		module = cfg[plugintype].keys()[0]
		self.writer = self.getplugin(plugintype,'--',0,module)
		if self.writer:
			self.writer.errors = []
			self.writer.p = self
			# merge defaults with configuration
			for k in self.writer.cfg.keys():
				if k in cfg[plugintype][module] and cfg[plugintype][module][k]: continue
				cfg[plugintype][module][k] = self.writer.cfg[k]
				if cfg[plugintype][module][k] is None:
					self.errors.append([module,'missing','[%s] \"%s=\"'%(module,k)])
					logger.error('%s\t%s',*self.errors[-1][1:])
				elif k[:4] == 'args' and cf.isinteger(k[4:]):
					if not 'args' in cfg[plugintype][module]: cfg[plugintype][module]['args'] = {}
					cfg[plugintype][module]['args'][int(k[4:])] = cfg[plugintype][module][k]
					del(cfg[plugintype][module][k])
				elif k[:7] == 'engines' and cf.isinteger(k[7:]):
					if not ',' in cfg[plugintype][module][k]: continue
					if not 'engines' in cfg[plugintype][module]: cfg[plugintype][module]['engines'] = {}
					cfg[plugintype][module]['engines'][int(k[7:])] = cfg[plugintype][module][k]
					del(cfg[plugintype][module][k])
			# convert writer config values to proper types
			self.__setDataTypes(self.writer.cfg,cfg[plugintype][module])
		else:
			self.errors.append([module,'failed','%s'%(module)])
			logger.error('%s\t%s',*self.errors[-1][1:])

		# initialize toolchain plugins
		for plugintype in plugintypes:
			self.toolchain[plugintype] = {}
			for lang in cfg[plugintype].keys():
				self.toolchain[plugintype][lang] = {}
				for sequence in sorted(map(int,cfg[plugintype][lang].keys())):
					for module in cfg[plugintype][lang][sequence]:
						self.toolchain[plugintype][lang][sequence] = self.getplugin(plugintype,lang,sequence,module)
						if self.toolchain[plugintype][lang][sequence]:
							self.toolchain[plugintype][lang][sequence].errors = []
							self.toolchain[plugintype][lang][sequence].p = self
							for k in self.toolchain[plugintype][lang][sequence].cfg.keys():
								if k in cfg[plugintype][lang][sequence][module] and cfg[plugintype][lang][sequence][module][k]: continue
								cfg[plugintype][lang][sequence][module][k] = self.toolchain[plugintype][lang][sequence].cfg[k]
								if cfg[plugintype][lang][sequence][module][k] is None:
									self.errors.append([module,'missing','[%s,%s,%s] \"%s=\"'%(lang,sequence,module,k)])
									logger.error('%s\t%s',*self.errors[-1][1:])
								elif k[:4] == 'args' and cf.isinteger(k[4:]):
									if not 'args' in cfg[plugintype][lang][sequence][module]: cfg[plugintype][lang][sequence][module]['args'] = {}
									cfg[plugintype][lang][sequence][module]['args'][int(k[4:])] = cfg[plugintype][lang][sequence][module][k]
									del(cfg[plugintype][lang][sequence][module][k])
								elif k[:7] == 'engines' and cf.isinteger(k[7:]):
									if not ',' in cfg[plugintype][lang][sequence][module][k]: continue
									if not 'engines' in cfg[plugintype][lang][sequence][module]: cfg[plugintype][lang][sequence][module]['engines'] = {}
									cfg[plugintype][lang][sequence][module]['engines'][int(k[7:])] = cfg[plugintype][lang][sequence][module][k]
									del(cfg[plugintype][lang][sequence][module][k])
							self.__setDataTypes(self.toolchain[plugintype][lang][sequence].cfg,cfg[plugintype][lang][sequence][module])
						else:
							self.errors.append([module,'failed','%s'%(module)])
							logger.error('%s\t%s',*self.errors[-1][1:])

		self.isloaded = not self.errors
		if self.isloaded:
			logger.debug('%s\t%s',*['success','loaded'])
		else:
			logger.error('%s\t%s',*['failed','not loaded'])

		return self.errors

	def unload(self):

		if not self.isloaded:
			self.errors.append([__name__,'skip','not loaded'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		# delete plugins
		for plugintype in plugintypes:
			for lang in self.toolchain[plugintype].keys():
				for sequence in sorted(self.toolchain[plugintype][lang].keys()):
					self.toolchain[plugintype][lang][sequence].errors = []
					self.toolchain[plugintype][lang][sequence].p = object
					del(self.toolchain[plugintype][lang][sequence])
				del(self.toolchain[plugintype][lang])

		# delete reader plugin
		self.reader.p = object
		self.reader.errors = []
		self.reader = object
		# delete writer plugin
		self.writer.p = object
		self.writer.errors = []
		self.writer = object

		# reset output dictionaries
		for k in self.cfextract.keys():
			del(self.cfextract[k])
		del(self.cfextract)
		self.cfextract = self.CustomExtract()
		for k in self.cferror.keys():
			del(self.cferror[k])
		del(self.cferror)
		self.cferror = self.CustomError()
		for k in self.cfoutput.keys():
			del(self.cfoutput[k])
		del(self.cfoutput)
		self.cfoutput = self.CustomOutput()
		# reset input dictionary
		for k in self.cfinput.keys():
			del(self.cfinput[k])
		del(self.cfinput)
		self.cfinput = self.CustomInput()

		# delete tempdir
		try:
			shutil.rmtree(self.tempdir)
			os.removedirs(os.path.dirname(self.tempdir))
		except:
			pass

		global me
		del(me)

		self.isloaded = False
		logger.debug('%s\t%s',*['success','unloaded'])

		return self.errors

	def open(self,cfg,verbose=True):

		if not self.isloaded:
			self.errors.append([__name__,'failed','not loaded'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		if self.isopen:
			self.errors.append([__name__,'skip','already open'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		if self.errors:
			self.isopen = False
			logger.error('%s\t%s',*['skip','errors on open'])

		self.graphname = cfg['graphname']
		self.graphs = os.path.abspath(os.path.expanduser(cfg['graphs']))
		self.plugins = os.path.abspath(os.path.expanduser(cfg['plugins']))

		self.rootfolder = self.reader.normalizeroot(os.path.abspath(os.path.expanduser(cfg['rootfolder'])))
		self._replace = {
			'%(rootfolder)s': self.rootfolder,
			'%(configpath)s': cf.getconfigpath(),
			'%(tempdir)s': self.tempdir,
			}
		self.superdomains = list(cfg['superdomains'])
		self.domains = list(cfg['domains'])
		self.subdomains = list(cfg['subdomains'])
		self.corpustypes = list(cfg['corpustypes'])
		self.filespec = (cfg['filespec'])
		self.lmonly = cfg['lmonly']
		self.bypass = cfg['bypass']
		self.distinct_flush_runs = cfg['distinct-flush-runs']
		self.queueby = cfg['queueby'] if cfg['queueby'] in ['source','target','both'] else 'source'

		# update language information
		if self.languagepairs:
			self.sourcepatterns, self.targetpatterns = zip(*self.languagepairs)
			self.sourcepatterns = cf.dedupelist([item.lower() for item in self.sourcepatterns])
			self.targetpatterns = cf.dedupelist([item.lower() for item in self.targetpatterns]+self.sourcepatterns)
			self.sourcelanguages = cf.dedupelist([lang for lang in self.sourcepatterns if not '*' in lang and not '?' in lang])
			self.languages = cf.dedupelist([lang for lang in self.sourcepatterns+self.targetpatterns if not '*' in lang and not '?' in lang])
		elif cfg['languagepairs']:
			for i in reversed(range(len(cfg['languagepairs']))):
				cfg['languagepairs'][i] = list(cfg['languagepairs'][i])
				for j,lang in enumerate(cfg['languagepairs'][i]):
					if lang.startswith('%(') and lang.endswith(')s'):
						cfg['languagepairs'][i][j] = ''
				if not '' in cfg['languagepairs'][i]:
					self.languagepairs.append(tuple(cfg['languagepairs'][i]))
			self.sourcepatterns, self.targetpatterns = zip(*self.languagepairs)
			self.sourcepatterns = cf.dedupelist([item.lower() for item in self.sourcepatterns])
			self.targetpatterns = cf.dedupelist([item.lower() for item in self.targetpatterns]+self.sourcepatterns)
			self.sourcelanguages = cf.dedupelist([lang for lang in self.sourcepatterns if not '*' in lang and not '?' in lang])
			self.languages = cf.dedupelist([lang for lang in self.sourcepatterns+self.targetpatterns if not '*' in lang and not '?' in lang])
		else:
			self.sourcelanguages = []
			self.languages = []
			self.sourcepatterns = []
			self.targetpatterns = []

		if cfg['versions']:
			# report versions information
			sys.stderr.write('%s: %s\n'%(__name__,cfg['version']))
			sys.stderr.write('%s: %s\n'%(self.reader.__module__,self.reader.cfg['version']))
			sys.stderr.write('%s: %s\n'%(self.writer.__module__,self.writer.cfg['version']))
			langs = {}
			for plugintype in plugintypes:
				for lang in self.toolchain[plugintype]:
					for sequence in self.toolchain[plugintype][lang]:
#						module = ','.join(self.toolchain[plugintype][lang][sequence].__module__.split(',')[-2:])
						sys.stderr.write('%s: %s\n'%(self.toolchain[plugintype][lang][sequence].__module__,self.toolchain[plugintype][lang][sequence].cfg['version']))

			self.isopen = False

		else:
			# display status information
			if not self.errors and verbose:
				sys.stdout.write('Graph: %s\n'%(self.graphname))
				sys.stdout.write("   language(s):")
				sys.stdout.flush()

			# replace remaining escaped values in configuration, e.g. %(rootfolder)s
			for k in cfg['reader'][cfg['reader'].keys()[0]].keys():
				cfg['reader'][cfg['reader'].keys()[0]][k] = self.__filter(cfg['reader'][cfg['reader'].keys()[0]][k])
			# open the reader plugin
			self.reader.open(self,cfg['reader'][cfg['reader'].keys()[0]])
			self.errors.extend(self.reader.errors)
			self.reader.isopen = not self.reader.errors

			# replace remaining escaped values in configuration, e.g. %(rootfolder)s
			for k in cfg['writer'][cfg['writer'].keys()[0]].keys():
				cfg['reader'][cfg['reader'].keys()[0]][k] = self.__filter(cfg['writer'][cfg['writer'].keys()[0]][k])
			# open the writer plugin
			self.writer.open(self,cfg['writer'][cfg['writer'].keys()[0]])
			self.errors.extend(self.writer.errors)
			self.writer.isopen = not self.writer.errors

			# filters & aligners
			langs = {}
			for plugintype in plugintypes:
				for lang in self.toolchain[plugintype]:
					if not lang in langs.keys() and not cfg['versions']:
						langs[lang] = None
						if not self.errors and verbose:
							sys.stdout.write(" %s"%(lang))
							sys.stdout.flush()
					for sequence in self.toolchain[plugintype][lang]:
						module = ','.join(self.toolchain[plugintype][lang][sequence].__module__.split(',')[-2:])
						# replace remaining escaped values in configuration values (not keys), e.g. %(rootfolder)s
						for k in cfg[plugintype][lang][sequence][module].keys():
							cfg[plugintype][lang][sequence][module][k] = self.__filter(cfg[plugintype][lang][sequence][module][k])
						# open the plugin
						self.toolchain[plugintype][lang][sequence].open(self,cfg[plugintype][lang][sequence][module])
						self.errors.extend(self.toolchain[plugintype][lang][sequence].errors)
						self.toolchain[plugintype][lang][sequence].isopen = not self.toolchain[plugintype][lang][sequence].errors

			if not self.errors and verbose:
				sys.stdout.write('\n')
				sys.stdout.flush()

			self.isopen = not self.errors
			if self.isopen:
				logger.debug('%s\t%s',*['success','opened'])
			else:
				logger.error('%s\t%s',*['failed','not opened'])

		return self.errors

	def close(self):
		'''Close all filters in the graph'''

		if self.isrunning:
			self.errors.append([__name__,'failed','running'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		if not self.isopen:
			self.errors.append([__name__,'skip','not open'])
			logger.warn('%s\t%s',*self.errors[-1][1:])
			return self.errors

		if self.errors:
			self.isopen = False
			logger.warn('%s\t%s',*['skip','errors'])

		# close language filters
		for plugintype in plugintypes:
			if self.errors: break
			for lang in [lang for lang in self.toolchain[plugintype] if not lang == 'alt']:
				if self.errors: break
				for sequence in sorted(self.toolchain[plugintype][lang].keys()):
					if self.errors: break
					try:
						self.toolchain[plugintype][lang][sequence].close()
						self.errors.extend(self.toolchain[plugintype][lang][sequence].errors)
						self.toolchain[plugintype][lang][sequence].isopen = False
					except Exception,e:
						if (os.name == 'nt' and e.errno == 13) or unicode(e) == 'list index out of range': continue
						self.errors.append([__name__,'failed','%s %s'%(','.join(self.toolchain[plugintype][lang][sequence].__module__.split(',')[1:]),unicode(e))])
						logger.error('%s\t%s',*self.errors[-1][1:])

		# close alt filters
		for plugintype in plugintypes:
			if self.errors: break
			for lang in [lang for lang in self.toolchain[plugintype] if lang == 'alt']:
				if self.errors: break
				for sequence in sorted(self.toolchain[plugintype][lang].keys()):
					if self.errors: break
					try:
						self.toolchain[plugintype][lang][sequence].close()
						self.errors.extend(self.toolchain[plugintype][lang][sequence].errors)
						self.toolchain[plugintype][lang][sequence].isopen = False
					except Exception,e:
						if (os.name == 'nt' and e.errno == 13) or unicode(e) == 'list index out of range': continue
						self.errors.append([__name__,'failed','%s %s'%(','.join(self.toolchain[plugintype][lang][sequence].__module__.split(',')[1:]),unicode(e))])
						logger.error(u'%s\t%s',*self.errors[-1][1:])

		# close reader
		if 'reader' in dir(self):
			try:
				self.reader.close()
			except Exception,e:
				if not (os.name == 'nt' and e.errno == 13):
					self.errors.append([__name__,'failed','%s %s'%(','.join(self.reader.__module__.split(',')[1:]),unicode(e))])
					logger.error('%s\t%s',*self.errors[-1][1:])
			finally:
				self.errors.extend(self.reader.errors)
				self.reader.isopen = False

		# close writer
		if 'writer' in dir(self):
			try:
				self.writer.close()
			except Exception,e:
				if not(os.name == 'nt' and e.errno == 13):
					self.errors.append([__name__,'failed','%s %s'%(','.join(self.writer.__module__.split(',')[1:]),unicode(e))])
					logger.error('%s\t%s',*self.errors[-1][1:])
			finally:
				self.errors.extend(self.writer.errors)
				self.writer.isopen = False

		self.isopen = False
		logger.debug('%s\t%s',*['success','closed'])

		return self.errors

	def run(self,filelist,run):
		'''run filtergraph'''

		# prepare to run filtergraph
		self.filelist = [filespec.strip().replace('/',os.sep).replace('\\',os.sep) for filespec in filelist]
		logger.debug('%s\t%s',*['start',os.sep.join(self.filelist)])

		# update languages after getting input file
		if self.reader.getrecord(): return -2
		for lang in self.languages:
			for plugintype in plugintypes:
				if lang in self.toolchain[plugintype]: continue
				self.toolchain[plugintype][lang] = {}

		# run reader class
		result = self.reader.run()
		if result or self.reader.errors:
			self.reader.flush()
			return result

		if not self.bypass:

			# run filters and aligners
			if not self.distinct_flush_runs or (self.distinct_flush_runs and run%2):
				result = self.__run()
				if result:
					for k in self.cfoutput.keys():
						del(self.cfoutput[k])
					for k in self.cferror.keys():
						del(self.cferror[k])
					return result

			# flush filters and aligners
			if not self.distinct_flush_runs or (self.distinct_flush_runs and not run%2):
				result = self.__flush()
				if result:
					for k in self.cfoutput.keys():
						del(self.cfoutput[k])
					for k in self.cferror.keys():
						del(self.cferror[k])
					return result

		# run writer class (save to temp files)
		self.writer.run(self.cfoutput)
		self.writer.run(self.cferror)

		# flush writer class (push temp files to storage)
		self.writer.flush(self.cfoutput)
		self.writer.flush(self.cferror)

		# flush reader class (release input record after finished)
		self.reader.flush()

		return result

	def decodepath(self,text,appendmodule=False):
		if text.strip(',').count(',') == 4:
			plugintype,language,sequence,location,module = text.strip(',').split(',')
		elif text.strip(',').count(',') == 3:
			plugintype,language,sequence,location,module = ['']+text.strip(',').split(',')
		elif text.strip(',').count(',') == 2:
			plugintype,sequence,language,location,module = ['']+['']+text.strip(',').split(',')
		elif text.strip(',').count(',') == 1:
			plugintype,language,sequence,location,module = ['']+['']+['']+text.strip(',').split(',')
		else:
			plugintype,language,sequence,location,module = ['']+['']+['']+['']+text.strip(',').split(',')

		if   location == '0':
			path = os.sep.join([self.plugins,module]) if appendmodule else self.plugins
		elif location == '1':
			path = os.sep.join([self.plugins,language,module]) if appendmodule else os.sep.join([self.plugins,language])
		elif location == '2':
			path = os.sep.join([self.graphs,self.graphname,module]) if appendmodule else os.sep.join([self.graphs,self.graphname])
		elif location == '3':
			path = os.sep.join([self.graphs,self.graphname,language,module]) if appendmodule else os.sep.join([self.graphs,self.graphname,language])
		elif location == '4':
			path = cf.whereis(module) if appendmodule else os.path.dirname(cf.whereis(module))
		else: #DEFAULT
			path = os.path.abspath(os.path.expanduser(module)) if appendmodule else os.path.dirname(os.path.abspath(os.path.expanduser((module))))
			path = path if os.path.exists(path) else ''
		return path

	def getplugin(self,plugintype,lang,sequence,module):
		# test if configured plugin exists
		kind = module.split(',')[1].split('-')[0] if '-' in module else module
		tmpmodule = module if kind in ['reader','writer'] else '%s,%s,%d,%s'%(plugintype,lang,sequence,module)
		origfile = '%s.py'%(self.decodepath(tmpmodule,True))
		if not os.path.exists(origfile):
			logger.error('%s\t%s',*['missing',origfile])
			return None
		# import plugin, copy plugin to temp folder and rename based on graph identiy
		shutil.copyfile(origfile,os.sep.join([self.tempdir,'%s.py'%(tmpmodule)]))
		fp,pathname,description = imp.find_module(tmpmodule,[self.tempdir])
		mod = imp.load_module(tmpmodule,fp,pathname,description)
		cffilter = mod.reader() if kind == 'reader' else mod.writer() if kind == 'writer' else mod.aligner() if kind == plugintypes[1] else mod.filter()
		return cffilter

	def getcorpusattributes(self,rootfolder='',roottype='',stage=''):
		if self.isopen:
			return self.reader.getcorpusattributes(self.rootfolder,self.roottype,self.stage[0])
		elif self.isloaded:
			return self.reader.getcorpusattributes(rootfolder,roottype,stage)
		else:
			return []

	def getallinputs(self,rootfolder='',roottype='',stage=''):
		if self.isopen:
			return self.reader.getallinputs(self.rootfolder,self.roottype,self.stage[0])
		elif self.isloaded:
			return self.reader.getallinputs(rootfolder,roottype,stage)
		else:
			return []

	def buildqueue(self,srcqueue,searchpaths=[]):
		# get root folder from reader plugin
		fs,root = self.reader.getroot(self.rootfolder)
		if searchpaths:
			searchpaths = [self.reader.path2list(qfile,root) for qfile in searchpaths]
		else:
			'''Converts ini named option= to levelsX='''
			# set fixed levels from top down
			levels = [[fs],[root]]
			# add ROOTTYPE
			levels.append([self.reader.roottype])
			# add stage
			if self.reader.stage: levels.append(self.reader.stage)
			# set middle levels
			if self.superdomains: levels.append(self.superdomains)
			if self.domains: levels.append(self.domains)
			if self.subdomains: levels.append(self.subdomains)
			# set bottom levels
			if self.languagepairs and self.corpustypes:
				newitems = {}
				# add corpustypes and languages to paths. note: expands corpustypes to avoid ambiguities.
				for corpustype in self.corpustypes:
					for path in cf.levels2lists(levels):
						if '*' in corpustype or '?' in corpustype:
							import glob
							ctypes = [ct for ct in [os.path.basename(c) for c in glob.glob('%s%s%s'%(os.sep.join(path),os.sep,corpustype)) if (
														os.path.basename(c).startswith('tm') or 
														os.path.basename(c).startswith('lm')) and 
															fnmatch.fnmatch(os.path.basename(c),corpustype)]]
						else:
							ctypes = [corpustype]
						for ctype in ctypes:
							for pair in self.languagepairs:

#								if self.p.queueby == 'source':
#								elif self.p.queueby == 'target':
#								elif self.p.queueby == 'both':

								if self.lmonly:
									if pair[0] == pair[1]:
										continue
									else:
										newitems[os.sep.join([ctype.strip(),pair[0].strip().replace('_','?'),pair[1].strip().replace('_','?')])] = None
								else:
									if ctype.strip().lower().startswith('tm'):
										newitems[os.sep.join([ctype.strip(),pair[0].strip().replace('_','?'),pair[0].strip().replace('_','?')])] = None
									else:
										newitems[os.sep.join([ctype.strip()]+[item.strip().replace('_','?') for item in pair])] = None
				if not newitems.keys: return 0
				levels.append(newitems.keys())
			# add filespec(s) to list
			levels.append(self.filespec)
			# convert list to glob paths
			searchpaths = [filelist[:-2]+filelist[-2].split(os.sep)+[filelist[-1]] for filelist in cf.levels2lists(levels)]
		return self.reader.buildqueue(srcqueue,searchpaths)

	def __run(self):
		self.prefilterdone = {}
		self.alignerdone = {}
		self.postfilterdone = {}

		try:
			alts = bool([k for k in self.cfoutput.keys() if not k[cf.kind].lower().startswith('tm') and not k[cf.kind].lower().startswith('lm')])
		except Exception,e:
			alts = True

		if not alts:
			# prefilter, all srclang
			testkeys = [k for k in self.prefilterdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[0]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].run(k)
						if result or self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].errors: return result
					self.prefilterdone[k] = None
					testkeys = sorted([k for k in self.prefilterdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))])
			# prefilter, all rdrlang
			testkeys = [k for k in self.prefilterdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[0]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].run(k)
						if result or self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].errors: return result
					self.prefilterdone[k] = None
					testkeys = [k for k in self.prefilterdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# filter aligner, 'tm' by rdrlang
			testkeys = [k for k in self.alignerdone.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[1]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[1]][k[cf.rdrlang]][sequence].run(k)
						if result or self.toolchain[plugintypes[1]][k[cf.rdrlang]][sequence].errors: return result
					self.alignerdone[k] = None
					testkeys = [k for k in self.alignerdone.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]
			# postfilter, all srclang
			testkeys = [k for k in self.postfilterdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[2]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].run(k)
						if result or self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].errors: return result
					self.postfilterdone[k] = None
					testkeys = [k for k in self.postfilterdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# postfilter, all rdrlang
			testkeys = [k for k in self.postfilterdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[2]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].run(k)
						if result or self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].errors: return result
					self.postfilterdone[k] = None
					testkeys = [k for k in self.postfilterdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]

		# filter, alt (no language)
		if 'alt' in self.toolchain[plugintypes[0]]:
			for k in sorted(self.cfoutput.keys()):
				for sequence in sorted(self.toolchain[plugintypes[0]]['alt'].keys()):
					result = self.toolchain[plugintypes[0]]['alt'][sequence].run(k)
					if result or self.toolchain[plugintypes[0]]['alt'][sequence].errors: return result

		return 0

	def __flush(self):
		self.preflushdone = {}
		self.alignerflushdone = {}
		self.postflushdone = {}

		try:
			alts = bool([k for k in self.cfoutput.keys() if not k[cf.kind].lower().startswith('tm') and not k[cf.kind].lower().startswith('lm')])
		except Exception,e:
			alts = True

		if not alts:
			testkeys = [k for k in self.preflushdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# prefilter, all srclang
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[0]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].flush(k)
						if result or self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].errors: return result
					self.preflushdone[k] = None
					testkeys = [k for k in self.preflushdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# prefilter, all rdrlang
			testkeys = [k for k in self.preflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[0]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].flush(k)
						if result or self.toolchain[plugintypes[0]][k[cf.rdrlang]][sequence].errors: return result
					self.preflushdone[k] = None
					testkeys = [k for k in self.preflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# filter aligner, 'tm' by rdrlang
			testkeys = [k for k in self.alignerflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[1]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[1]][k[cf.rdrlang]][sequence].flush(k)
						if result or self.toolchain[plugintypes[1]][k[cf.rdrlang]][sequence].errors: return result
					self.alignerflushdone[k] = None
					testkeys = [k for k in self.alignerflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and k[cf.kind].lower().startswith('tm')]
			# postfilter, all srclang
			testkeys = [k for k in self.postflushdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[2]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].flush(k)
						if result or self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].errors: return result
					self.postflushdone[k] = None
					testkeys = [k for k in self.postflushdone.keys() if k[cf.rdrlang] == k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			# postfilter, all rdrlang
			testkeys = [k for k in self.postflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]
			while not sorted(testkeys) == sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
				for k in sorted([k for k in self.cfoutput.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]):
					if k in testkeys: continue
					for sequence in sorted(self.toolchain[plugintypes[2]][k[cf.rdrlang]].keys()):
						result = self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].flush(k)
						if result or self.toolchain[plugintypes[2]][k[cf.rdrlang]][sequence].errors: return result
					self.postflushdone[k] = None
					testkeys = [k for k in self.postflushdone.keys() if k[cf.rdrlang] != k[cf.srclang] and (k[cf.kind].lower().startswith('tm') or k[cf.kind].lower().startswith('lm'))]

		# filter, alt (no language)
		if 'alt' in self.toolchain[plugintypes[0]]:
			for k in sorted(self.cfoutput.keys()):
				for sequence in sorted(self.toolchain[plugintypes[0]]['alt'].keys()):
					result = self.toolchain[plugintypes[0]]['alt'][sequence].flush(k)
					if result or self.toolchain[plugintypes[0]]['alt'][sequence].errors: return result

		return 0

	def __filter(self,text):
		'''replace escaped config values'''
		if type(text) is str or type(text) is unicode:
			for i, j in self._replace.items():
				text = text.replace(i, j)
		elif type(text) is dict:
			for k,txt in text.items():
				if type(txt) is str or type(txt) is unicode:
					for i, j in self._replace.items():
						text[k] = txt.replace(i, j)
		elif type(text) is list:
			for k,txt in enumerate(text):
				if type(txt) is str or type(txt) is unicode:
					for i, j in self._replace.items():
						text[k] = txt.replace(i, j)
		return text

	def __setDataTypes(self,template,dest):
		try:
			for k in dest.keys():
				if not k in template: continue
				if type(template[k]) is list:
					if not type(dest[k]) is list: dest[k] = dest[k].strip(', ').split(',')
				elif type(template[k]) is bool:
					if not type(dest[k]) is bool: dest[k] = cf.zbool(dest[k])
				elif type(template[k]) is int:
					if not type(dest[k]) is int and not dest[k].startswith('%(') and not dest[k].endswith(')s'): dest[k] = int(dest[k])
				elif type(template[k]) is float:
					if not type(dest[k]) is float: dest[k] = float(dest[k])
				elif type(template[k]) is dict:
					if not type(dest[k]) is dict:
						dest[k] = [pair.strip(' :').split(':') for pair in unicode(dest[k]).strip(';').split(';') if ':' in pair]
						dest[k] = dict([[key.lower(),value] for key,value in [pair if len(pair) == 2 else pair+[''] for pair in dest[k]]])
					dest[k] = dict(template[k],**dict(dest[k]))
					if k == 'kw':
						cf.setkwtypes(dest[k])
		except ValueError, e:
			self.errors.append([__name__,'invalid','incorrect value type: %s=%s (%s)'%(k, dest[k], e)])
			logger.error('%s\t%s',*self.errors[-1][1:])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import parser\n\tfrom %s import manager\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	os.path.splitext(os.path.basename(sys.argv[0]))[0],
	)

licensetxt=u'''CorpusFiltergraph™ v4.6
Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
