#! /usr/bin/env python
# -*- coding: utf8 -*-

#===============================================================================
# Author: Walapa Muangjeen
#===============================================================================

#version:
#4.6.344 - minor version update

import os
import sys
import string
import re
import common as cf
import logging

logger = logging.getLogger('.'.join([os.path.splitext(os.path.basename(sys.argv[0]))[0],'manager','filtergraph',__name__]))
skipclose = True
extended = u'€‚ƒ„…†‡ˆ‰Š‹Œ‘’“”•–—˜™›¡¢£¤¥¦§¨©ª«¬­®¯°±²³´µ¶·¸¹º»¼½¾¿÷˗⁻₋−：。、'
whitespace = u'  	\x0b\x0c         ​'

class filter(object):

	cfg = {
		'delete-line': False,
		'delete-characters': False,
		'irrelevant': string.digits+string.punctuation+whitespace,
		'exceptions': u'',
		'use-extended': True,
		'encoding': 'utf8',
		'inputfile': '',
		'outputfile': '',
		'version': '4.6.344',
		}
	delete_line = False
	delete_characters = False
	irrelevant = string.digits+string.punctuation+whitespace
	exceptions = ''
	encoding = 'utf8'
	inputfile = ''
	outputfile = ''
	isopen = False
	p = object
	errors = []

	def open(self,parent,cfg):
		self.encoding = 'utf8' if 'utf8' in cfg['encoding'].lower().replace('-','') else cfg['encoding']
		self.inputfile = cfg['inputfile'].replace('%(rootfolder)s',parent.rootfolder) if cfg['inputfile'] else self.inputfile
		self.outputfile = cfg['outputfile'].replace('%(rootfolder)s',parent.rootfolder) if cfg['outputfile'] else self.outputfile
		if (self.inputfile and not self.outputfile) or (not self.inputfile and self.outputfile):
			self.errors.append([__name__,'invalid','[%s] inputfile=%s without outputfile= value'%(__name__,cfg['inputfile'])])
			logger.warn('%s\t%s',*self.errors[-1][1:])

		self.delete_line = cfg['delete-line']
		self.delete_characters = cfg['delete-characters']
		self.irrelevant = self.irrelevant + extended if cfg['use-extended'] else self.irrelevant
		self.irrelevant = ''.join([ char for char in cf.dedupelist(cfg['irrelevant']) if not char in self.irrelevant ])+self.irrelevant
		self.exceptions = ''.join([ char for char in cf.dedupelist(cfg['exceptions']) if not char in self.exceptions ])+self.exceptions
		for char in self.exceptions:
			self.irrelevant = self.irrelevant.replace(char,'')

		self.regex = re.compile('[%s]'%re.escape(self.irrelevant))

	def run(self,k):
		global skipclose
		skipclose = not self.inputfile
		if self.inputfile: return

		if self.delete_line:
			self.p.cfoutput[k]['tempbuff'] = [line for line in self.p.cfoutput[k]['tempbuff'] if len(self.regex.sub('', line))]
		elif self.delete_characters:
			self.p.cfoutput[k]['tempbuff'] = [self.regex.sub('',line) for line in self.p.cfoutput[k]['tempbuff']]
		else:
			self.p.cfoutput[k]['tempbuff'] = ['' if not self.regex.sub('', line) else line for line in self.p.cfoutput[k]['tempbuff']]

	def flush(self,k):
		return

	def close(self):
		if skipclose: return

		import codecs
		from progress_bar import ProgressBar

		if not os.path.exists(self.inputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.inputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])
			return

		# make output folder
		try:
			os.makedirs(os.path.dirname(self.outputfile))
		except OSError,e:
			if not e.errno == 17:
				logger.exception('%s\t%s, %s, %s',*['failed',e.errno,e.strerror,e.filename,])
				raise OSError(e)

		# open input and output files
		out = self.outputfile
		if out == self.inputfile:
			import tempfile
			fd,out = tempfile.mkstemp(suffix='.tmp', prefix='~', dir=self.p.tempdir)
			os.close(fd)
		try:
			o = codecs.open(out,'w',self.encoding)
			i = codecs.open(self.inputfile,'r',self.encoding)
			ttl_lines = cf.file_length(self.inputfile)
		except:
			os.unlink(out)
			raise RuntimeError('Failed to open [%s] input/output files'%(__name__))

		progress = ProgressBar()
		progress.reset(0,ttl_lines,72,mode='fixed',char='#')
		sys.stderr.write('[%s] %s\n'%(__name__,self.outputfile.encode('utf8')))

		try:
			# loop writes output line-by-line
			if self.delete_line:
				for line in i:
					progress.increment_amount()
					sys.stderr.write(progress.bar)
					sys.stderr.flush()
					if not len(self.regex.sub('', line)): continue
					o.write('%s\n'%(line.rstrip('\r\n')))

			elif self.delete_characters:
				for line in i:
					progress.increment_amount()
					sys.stderr.write(progress.bar)
					sys.stderr.flush()
					o.write('%s\n'%(self.regex.sub('', line.rstrip('\r\n'))))

			else:
				for line in i:
					progress.increment_amount()
					sys.stderr.write(progress.bar)
					sys.stderr.flush()
					o.write('%s\n'%('' if not self.regex.sub('', line.rstrip('\r\n')) else line.rstrip('\r\n')))
			sys.stderr.write('\n')
			progress.update_amount(ttl_lines)
			sys.stderr.write(progress.bar)
			sys.stderr.write('\n')
			sys.stderr.flush()
			# close input and output files
			i.close()
			o.close()

			if not out == self.outputfile:
				import shutil
				shutil.move(out,self.outputfile)

		except KeyboardInterrupt:
			os.unlink(out)
			raise KeyboardInterrupt()

		if not os.path.exists(self.outputfile):
			self.errors.append([__name__,'missing','[%s] %s'%(__name__,self.outputfile)])
			logger.error('%s\t%s',*self.errors[-1][1:])

def usage():
	'''Command prompt help.'''
	return "\n%s\n\tUsage:\n\tfrom %s import filter\n"%(
	os.path.basename(sys.argv[0]),
	os.path.splitext(os.path.basename(sys.argv[0]))[0]
	)

licensetxt=u'''CorpusFiltergraph™ v4.6
Copyright © 2010-2013 Precision Translation Tools Co., Ltd.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see http://www.gnu.org/licenses/.

For more information, please contact Precision Translation Tools Co., Ltd.
at: http://www.precisiontranslationtools.com'''

if __name__ == "__main__":
	import os
	import sys
	sys.stdout.write(usage().encode('utf8')+'\n')
